@@ -1,9 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2011,2013-2015                      Université de Bordeaux
- * Copyright (C) 2012,2013                                Inria
- * Copyright (C) 2010                                     Mehdi Juhoor
- * Copyright (C) 2010-2013,2015,2017                      CNRS
+ * Copyright (C) 2009-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010       Mehdi Juhoor
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -39,7 +37,7 @@
 
 #include <starpu.h>
 
-static float *A, *B, *C;
+static float *A, *B, *C, *Cref;
 static starpu_data_handle_t A_handle, B_handle, C_handle;
 
 static unsigned nslicesx = 4;
@@ -54,26 +52,29 @@ static unsigned ydim = 1024;
 static unsigned zdim = 512;
 #endif
 
+extern void hip_mult(void *descr[], void *arg);
+extern void cuda_mult(void *descr[], void *arg);
 
 /*
- * That program should compute C = A * B 
- * 
+ * That program should compute C = A * B
+ *
  *   A of size (z,y)
  *   B of size (x,z)
  *   C of size (x,y)
-
-              |---------------|
-            z |       B       |
-              |---------------|
-       z              x
-     |----|   |---------------|
-     |    |   |               |
-     |    |   |               |
-     | A  | y |       C       |
-     |    |   |               |
-     |    |   |               |
-     |----|   |---------------|
-
+ *
+ *             |---------------|
+ *           z |       B       |
+ *             |---------------|
+ *      z              x
+ *    |----|   |---------------|
+ *    |    |   |               |
+ *    |    |   |               |
+ *    | A  | y |       C       |
+ *    |    |   |               |
+ *    |    |   |               |
+ *    |----|   |---------------|
+ *
+ * Note: we use FORTRAN ordering.
  */
 
 /*
@@ -90,14 +91,20 @@ void cpu_mult(void *descr[], void *arg)
 	uint32_t nxC, nyC, nyA;
 	uint32_t ldA, ldB, ldC;
 
-	/* .blas.ptr gives a pointer to the first element of the local copy */
+	/* ptr gives a pointer to the first element of the local copy */
 	subA = (float *)STARPU_MATRIX_GET_PTR(descr[0]);
 	subB = (float *)STARPU_MATRIX_GET_PTR(descr[1]);
 	subC = (float *)STARPU_MATRIX_GET_PTR(descr[2]);
 
-	/* .blas.nx is the number of rows (consecutive elements) and .blas.ny
-	 * is the number of lines that are separated by .blas.ld elements (ld
-	 * stands for leading dimension).
+	/*
+	 * Note: STARPU_MATRIX_GET_NX/NY is different from X/Y of the FORTRAN
+	 * ordering:
+	 * - nx is the number of consecutive elements (thus the number of rows
+	 *   in FORTRAN order)
+	 * - ny is the number of series that are separated by ld elements (thus
+	 *   the number of columns in FORTRAN order)
+	 * - ld stands for leading dimension
+	 *
 	 * NB: in case some filters were used, the leading dimension is not
 	 * guaranteed to be the same in main memory (on the original matrix)
 	 * and on the accelerator! */
@@ -109,11 +116,11 @@ void cpu_mult(void *descr[], void *arg)
 	ldB = STARPU_MATRIX_GET_LD(descr[1]);
 	ldC = STARPU_MATRIX_GET_LD(descr[2]);
 
-	/* we assume a FORTRAN-ordering! */
+	/* we use a FORTRAN-ordering! */
 	unsigned i,j,k;
-	for (i = 0; i < nyC; i++)
+	for (i = 0; i < nyC; i++) /* iterate over columns of C */
 	{
-		for (j = 0; j < nxC; j++)
+		for (j = 0; j < nxC; j++) /* iterate over rows of C */
 		{
 			float sum = 0.0;
 
@@ -133,9 +140,15 @@ static void init_problem_data(void)
 
 	/* we initialize matrices A, B and C in the usual way */
 
-	A = (float *) malloc(zdim*ydim*sizeof(float));
-	B = (float *) malloc(xdim*zdim*sizeof(float));
-	C = (float *) malloc(xdim*ydim*sizeof(float));
+	starpu_malloc_flags((void **)&A, zdim*ydim*sizeof(float), STARPU_MALLOC_PINNED|STARPU_MALLOC_SIMULATION_FOLDED);
+	starpu_malloc_flags((void **)&B, xdim*zdim*sizeof(float), STARPU_MALLOC_PINNED|STARPU_MALLOC_SIMULATION_FOLDED);
+	starpu_malloc_flags((void **)&C, xdim*ydim*sizeof(float), STARPU_MALLOC_PINNED|STARPU_MALLOC_SIMULATION_FOLDED);
+
+	Cref = (float *) malloc(xdim*ydim*sizeof(float));
+	assert(A);
+	assert(B);
+	assert(C);
+	assert(Cref);
 
 	/* fill the A and B matrices */
 	starpu_srand48(2009);
@@ -160,6 +173,7 @@ static void init_problem_data(void)
 		for (i=0; i < xdim; i++)
 		{
 			C[j+i*ydim] = (float)(0);
+			Cref[j+i*ydim] = (float)(0);
 		}
 	}
 }
@@ -168,7 +182,7 @@ static void partition_mult_data(void)
 {
 	/* note that we assume a FORTRAN ordering here! */
 
-	/* The BLAS data interface is described by 4 parameters: 
+	/* The BLAS data interface is described by 4 parameters:
 	 *  - the location of the first element of the matrix to monitor (3rd
 	 *    argument)
 	 *  - the number of elements between columns, aka leading dimension
@@ -179,13 +193,13 @@ static void partition_mult_data(void)
 	 * The first elements is a pointer to the data_handle that will be
 	 * associated to the matrix, and the second elements gives the memory
 	 * node in which resides the matrix: 0 means that the 3rd argument is
-	 * an adress in main memory.
+	 * an address in main memory.
 	 */
-	starpu_matrix_data_register(&A_handle, STARPU_MAIN_RAM, (uintptr_t)A, 
+	starpu_matrix_data_register(&A_handle, STARPU_MAIN_RAM, (uintptr_t)A,
 		ydim, ydim, zdim, sizeof(float));
-	starpu_matrix_data_register(&B_handle, STARPU_MAIN_RAM, (uintptr_t)B, 
+	starpu_matrix_data_register(&B_handle, STARPU_MAIN_RAM, (uintptr_t)B,
 		zdim, zdim, xdim, sizeof(float));
-	starpu_matrix_data_register(&C_handle, STARPU_MAIN_RAM, (uintptr_t)C, 
+	starpu_matrix_data_register(&C_handle, STARPU_MAIN_RAM, (uintptr_t)C,
 		ydim, ydim, xdim, sizeof(float));
 
 	/* A filter is a method to partition a data into disjoint chunks, it is
@@ -216,9 +230,9 @@ static void partition_mult_data(void)
  *	the number of filters to apply, and the indexes for each filters, for
  *	instance:
  *
- *		A' handle is starpu_data_get_sub_data(A_handle, 1, 1); 
- *		B' handle is starpu_data_get_sub_data(B_handle, 1, 2); 
- *		C' handle is starpu_data_get_sub_data(C_handle, 2, 2, 1); 
+ *		A' handle is starpu_data_get_sub_data(A_handle, 1, 1);
+ *		B' handle is starpu_data_get_sub_data(B_handle, 1, 2);
+ *		C' handle is starpu_data_get_sub_data(C_handle, 2, 2, 1);
  *
  *	Note that here we applied 2 filters recursively onto C.
  *
@@ -238,9 +252,9 @@ static void partition_mult_data(void)
  *		     |----|   |---|---|---|---|
  *		       A              C
  *
- *	IMPORTANT: applying filters is equivalent to partitionning a piece of
+ *	IMPORTANT: applying filters is equivalent to partitioning a piece of
  *	data in a hierarchical manner, so that memory consistency is enforced
- *	for each of the elements independantly. The tasks should therefore NOT
+ *	for each of the elements independently. The tasks should therefore NOT
  *	access inner nodes (eg. one column of C or the whole C) but only the
  *	leafs of the tree (ie. blocks here). Manipulating inner nodes is only
  *	possible by disapplying the filters (using starpu_data_unpartition), to
@@ -268,16 +282,25 @@ static struct starpu_perfmodel mult_perf_model =
 
 static struct starpu_codelet cl =
 {
-        /* we can only execute that kernel on a CPU yet */
-        /* CPU implementation of the codelet */
-        .cpu_funcs = {cpu_mult},
-        .cpu_funcs_name = {"cpu_mult"},
-        /* the codelet manipulates 3 buffers that are managed by the
-         * DSM */
-        .nbuffers = 3,
+	/* CPU implementation of the codelet */
+	.cpu_funcs = {cpu_mult},
+	.cpu_funcs_name = {"cpu_mult"},
+#ifdef STARPU_USE_HIP
+	/* HIP implementation of the codelet */
+	.hip_funcs = {hip_mult},
+	.hip_flags = {STARPU_HIP_ASYNC},
+#endif
+#ifdef STARPU_USE_CUDA
+	/* CUDA implementation of the codelet */
+	.cuda_funcs = {cuda_mult},
+        .cuda_flags = {STARPU_CUDA_ASYNC},
+	.where     = STARPU_CUDA,
+#endif
+	/* the codelet manipulates 3 buffers that are managed by the DSM */
+	.nbuffers = 3,
 	.modes = {STARPU_R, STARPU_R, STARPU_W},
-        /* in case the scheduling policy may use performance models */
-        .model = &mult_perf_model
+	/* in case the scheduling policy may use performance models */
+	.model = &mult_perf_model
 };
 
 static int launch_tasks(void)
@@ -286,14 +309,14 @@ static int launch_tasks(void)
 	/* partition the work into slices */
 	unsigned taskx, tasky;
 
-	for (taskx = 0; taskx < nslicesx; taskx++) 
+	for (taskx = 0; taskx < nslicesx; taskx++)
 	{
 		for (tasky = 0; tasky < nslicesy; tasky++)
 		{
 			/* C[taskx, tasky] = A[tasky] B[taskx] */
 
 			/* by default, starpu_task_create() returns an
- 			 * asynchronous task (ie. task->synchronous = 0) */
+			 * asynchronous task (ie. task->synchronous = 0) */
 			struct starpu_task *task = starpu_task_create();
 
 			/* this task implements codelet "cl" */
@@ -303,7 +326,7 @@ static int launch_tasks(void)
 			 *              |---|---|---|---|
 			 *              |   | * |   |   | B
 			 *              |---|---|---|---|
-			 *                    X 
+			 *                    X
 			 *     |----|   |---|---|---|---|
 			 *     |****| Y |   |***|   |   |
 			 *     |****|   |   |***|   |   |
@@ -342,6 +365,24 @@ static int launch_tasks(void)
 	return 0;
 }
 
+void check_result(float* C_gpu, float* C_ref, uint32_t ldC)
+{
+	unsigned i,j;
+	for (i = 0; i < ydim; i++)
+	{
+		for (j = 0; j < xdim; j++)
+		{
+			if(C_gpu[j + i*ldC]-C_ref[j + i*ldC] > 1e-6*C_ref[j + i*ldC])
+			{
+				printf("| Cref[%u,%u]=%f - Cgpu[%u,%u]=%f | Error in the computation of C: the difference between the two is bigger than 1e-6 * the reference"
+				       , i, j, C_ref[j + i*ldC], i, j, C_gpu[j + i*ldC]);
+				exit(1);
+			}
+		}
+	}
+	printf("SUCCESSFUL COMPUTATION\n");
+}
+
 int main(void)
 {
 	int ret;
@@ -356,18 +397,42 @@ int main(void)
 	init_problem_data();
 
 	/* partition matrices into blocks that can be manipulated by the
- 	 * codelets */
+	 * codelets */
 	partition_mult_data();
 
 	/* submit all tasks in an asynchronous fashion */
 	ret = launch_tasks();
 	if (ret == -ENODEV) goto enodev;
 
+	/* cpu compution to check */
+	/* ============================================= */
+
+	uint32_t ldA = ydim;
+	uint32_t ldB = zdim;
+	uint32_t ldC = ydim;
+
+	unsigned i,j,k;
+	for (i = 0; i < ydim; i++)
+	{
+		for (j = 0; j < xdim; j++)
+		{
+			float sum = 0.0;
+
+			for (k = 0; k < zdim; k++)
+			{
+				sum += A[j+k*ldA]*B[k+i*ldB];
+			}
+
+			Cref[j + i*ldC] = sum;
+		}
+	}
+
+	/* ============================================= */
 	/* wait for termination */
-        starpu_task_wait_for_all();
+	starpu_task_wait_for_all();
 
 	/* remove the filters applied by the means of starpu_data_map_filters; now
- 	 * it's not possible to manipulate a subset of C using starpu_data_get_sub_data until
+	 * it's not possible to manipulate a subset of C using starpu_data_get_sub_data until
 	 * starpu_data_map_filters is called again on C_handle.
 	 * The second argument is the memory node where the different subsets
 	 * should be reassembled, 0 = main memory (RAM) */
@@ -375,7 +440,7 @@ int main(void)
 	starpu_data_unpartition(B_handle, STARPU_MAIN_RAM);
 	starpu_data_unpartition(C_handle, STARPU_MAIN_RAM);
 
-	/* stop monitoring matrix C : after this, it is not possible to pass C 
+	/* stop monitoring matrix C : after this, it is not possible to pass C
 	 * (or any subset of C) as a codelet input/output. This also implements
 	 * a barrier so that the piece of data is put back into main memory in
 	 * case it was only available on a GPU for instance. */
@@ -383,9 +448,14 @@ int main(void)
 	starpu_data_unregister(B_handle);
 	starpu_data_unregister(C_handle);
 
-	free(A);
-	free(B);
-	free(C);
+	/* Comment to remove printing of results */
+	check_result(C, Cref, ldC);
+
+        starpu_free_flags(A, zdim*ydim*sizeof(float), STARPU_MALLOC_PINNED|STARPU_MALLOC_SIMULATION_FOLDED);
+	starpu_free_flags(B, xdim*zdim*sizeof(float), STARPU_MALLOC_PINNED|STARPU_MALLOC_SIMULATION_FOLDED);
+	starpu_free_flags(C, xdim*ydim*sizeof(float), STARPU_MALLOC_PINNED|STARPU_MALLOC_SIMULATION_FOLDED);
+
+	free(Cref);
 
 	starpu_shutdown();
 
@@ -395,4 +465,3 @@ enodev:
 	starpu_shutdown();
 	return 77;
 }
-
