@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -20,12 +20,12 @@
 #include <starpu.h>
 #include <common/blas.h>
 
-#define TAG11(k)	((starpu_tag_t)( (1ULL<<60) | (unsigned long long)(k)))
-#define TAG12(k,i)	((starpu_tag_t)(((2ULL<<60) | (((unsigned long long)(k))<<32)	\
+#define TAG_GETRF(k)	((starpu_tag_t)((1ULL<<60) | (unsigned long long)(k)))
+#define TAG_TRSM_LL(k,i)	((starpu_tag_t)(((2ULL<<60) | (((unsigned long long)(k))<<32)	\
 					| (unsigned long long)(i))))
-#define TAG21(k,j)	((starpu_tag_t)(((3ULL<<60) | (((unsigned long long)(k))<<32)	\
+#define TAG_TRSM_RU(k,j)	((starpu_tag_t)(((3ULL<<60) | (((unsigned long long)(k))<<32)	\
 					| (unsigned long long)(j))))
-#define TAG22(k,i,j)	((starpu_tag_t)(((4ULL<<60) | ((unsigned long long)(k)<<32) 	\
+#define TAG_GEMM(k,i,j)	((starpu_tag_t)(((4ULL<<60) | ((unsigned long long)(k)<<32) 	\
 					| ((unsigned long long)(i)<<16)	\
 					| (unsigned long long)(j))))
 #define PIVOT(k,i)	((starpu_tag_t)(((5ULL<<60) | (((unsigned long long)(k))<<32)	\
@@ -34,8 +34,8 @@
 #define FPRINTF(ofile, fmt, ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ## __VA_ARGS__); }} while(0)
 #define PRINTF(fmt, ...) do { if (!getenv("STARPU_SSILENT")) {printf(fmt, ## __VA_ARGS__); }} while(0)
 
-#define BLAS3_FLOP(n1,n2,n3)    \
-        (2*((uint64_t)n1)*((uint64_t)n2)*((uint64_t)n3))
+#define BLAS3_FLOP(n1,n2,n3) \
+	(2*((uint64_t)n1)*((uint64_t)n2)*((uint64_t)n3))
 
 #ifdef CHECK_RESULTS
 static void compare_A_LU(float *A, float *LU, unsigned size, unsigned ld)
@@ -68,15 +68,15 @@ static void compare_A_LU(float *A, float *LU, unsigned size, unsigned ld)
 		}
 	}
 
-        /* now A_err = L, compute L*U */
+	/* now A_err = L, compute L*U */
 	STARPU_STRMM("R", "U", "N", "U", size, size, 1.0f, U, size, L, size);
 
 	float max_err = 0.0f;
 	for (i = 0; i < size ; i++)
 	{
-		for (j = 0; j < size; j++) 
+		for (j = 0; j < size; j++)
 		{
-			max_err = STARPU_MAX(max_err, fabs(  L[j+i*size] - A[j+i*ld]  ));
+			max_err = STARPU_MAX(max_err, fabs(L[j+i*size] - A[j+i*ld]));
 		}
 	}
 
@@ -84,35 +84,39 @@ static void compare_A_LU(float *A, float *LU, unsigned size, unsigned ld)
 }
 #endif /* CHECK_RESULTS */
 
-void dw_cpu_codelet_update_u11(void **, void *);
-void dw_cpu_codelet_update_u12(void **, void *);
-void dw_cpu_codelet_update_u21(void **, void *);
-void dw_cpu_codelet_update_u22(void **, void *);
+void dw_cpu_codelet_update_getrf(void **, void *);
+void dw_cpu_codelet_update_trsm_ll(void **, void *);
+void dw_cpu_codelet_update_trsm_ru(void **, void *);
+void dw_cpu_codelet_update_gemm(void **, void *);
 
 #ifdef STARPU_USE_CUDA
-void dw_cublas_codelet_update_u11(void *descr[], void *_args);
-void dw_cublas_codelet_update_u12(void *descr[], void *_args);
-void dw_cublas_codelet_update_u21(void *descr[], void *_args);
-void dw_cublas_codelet_update_u22(void *descr[], void *_args);
+void dw_cublas_codelet_update_getrf(void *descr[], void *_args);
+void dw_cublas_codelet_update_trsm_ll(void *descr[], void *_args);
+void dw_cublas_codelet_update_trsm_ru(void *descr[], void *_args);
+void dw_cublas_codelet_update_gemm(void *descr[], void *_args);
 #endif
 
-void dw_callback_codelet_update_u11(void *);
-void dw_callback_codelet_update_u12_21(void *);
-void dw_callback_codelet_update_u22(void *);
+void dw_callback_codelet_update_getrf(void *);
+void dw_callback_codelet_update_trsm_ll_21(void *);
+void dw_callback_codelet_update_gemm(void *);
 
-void dw_callback_v2_codelet_update_u11(void *);
-void dw_callback_v2_codelet_update_u12(void *);
-void dw_callback_v2_codelet_update_u21(void *);
-void dw_callback_v2_codelet_update_u22(void *);
+void dw_callback_v2_codelet_update_getrf(void *);
+void dw_callback_v2_codelet_update_trsm_ll(void *);
+void dw_callback_v2_codelet_update_trsm_ru(void *);
+void dw_callback_v2_codelet_update_gemm(void *);
 
-extern struct starpu_perfmodel model_11;
-extern struct starpu_perfmodel model_12;
-extern struct starpu_perfmodel model_21;
-extern struct starpu_perfmodel model_22;
+extern struct starpu_perfmodel model_getrf;
+extern struct starpu_perfmodel model_trsm_ll;
+extern struct starpu_perfmodel model_trsm_ru;
+extern struct starpu_perfmodel model_gemm;
 extern unsigned bound;
 extern unsigned bounddeps;
 extern unsigned boundprio;
 
+extern starpu_data_handle_t scratch;
+void lu_kernel_init(int nb);
+void lu_kernel_fini(void);
+
 struct piv_s
 {
 	unsigned *piv; /* complete pivot array */
