@@ -27,6 +27,7 @@
 
 #include <limits.h>
 #include <string.h>
+#include <unistd.h>
 #include <math.h>
 #include <sys/types.h>
 #include <starpu.h>
@@ -44,9 +45,23 @@ static const TYPE m1 = -1.0;
 static const TYPE v0 = 0.0;
 #endif
 
+#ifdef STARPU_USE_HIP
+#include <hip/hip_runtime.h>
+#include <starpu_hipblas.h>
+static const TYPE p1 = 1.0;
+static const TYPE m1 = -1.0;
+static const TYPE v0 = 0.0;
+#endif
+
+#ifdef STARPU_QUICK_CHECK
+static unsigned niter = 2;
+#else
 static unsigned niter = 10;
+#endif
+static unsigned nsleeps = 1;
 static unsigned nslicesx = 4;
 static unsigned nslicesy = 4;
+static unsigned nslicesz = 4;
 #if defined(STARPU_QUICK_CHECK) && !defined(STARPU_SIMGRID)
 static unsigned xdim = 256;
 static unsigned ydim = 256;
@@ -56,14 +71,17 @@ static unsigned xdim = 960*4;
 static unsigned ydim = 960*4;
 static unsigned zdim = 960*4;
 #endif
+static unsigned size_set = 0;
 static unsigned check = 0;
 static unsigned bound = 0;
+static unsigned print_hostname = 0;
+static unsigned tiled = 0;
 
 static TYPE *A, *B, *C;
 static starpu_data_handle_t A_handle, B_handle, C_handle;
 
 #define FPRINTF(ofile, fmt, ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ## __VA_ARGS__); }} while(0)
-#define PRINTF(fmt, ...) do { if (!getenv("STARPU_SSILENT")) {printf(fmt, ## __VA_ARGS__); }} while(0)
+#define PRINTF(fmt, ...) do { if (!getenv("STARPU_SSILENT")) {printf(fmt, ## __VA_ARGS__); fflush(stdout); }} while(0)
 
 #ifdef STARPU_HAVE_BLAS
 static int check_output(void)
@@ -132,6 +150,8 @@ static void init_problem_data(void)
 
 static void partition_mult_data(void)
 {
+	unsigned x, y, z;
+
 	starpu_matrix_data_register(&A_handle, STARPU_MAIN_RAM, (uintptr_t)A,
 		ydim, ydim, zdim, sizeof(TYPE));
 	starpu_matrix_data_register(&B_handle, STARPU_MAIN_RAM, (uintptr_t)B,
@@ -149,14 +169,51 @@ static void partition_mult_data(void)
 	horiz.filter_func = starpu_matrix_filter_block;
 	horiz.nchildren = nslicesy;
 
-	starpu_data_partition(B_handle, &vert);
-	starpu_data_partition(A_handle, &horiz);
+	if (tiled)
+	{
+		struct starpu_data_filter vertA;
+		memset(&vertA, 0, sizeof(vertA));
+		vertA.filter_func = starpu_matrix_filter_vertical_block;
+		vertA.nchildren = nslicesz;
 
-	starpu_data_map_filters(C_handle, 2, &vert, &horiz);
+		struct starpu_data_filter horizB;
+		memset(&horizB, 0, sizeof(horizB));
+		horizB.filter_func = starpu_matrix_filter_block;
+		horizB.nchildren = nslicesz;
+
+		starpu_data_map_filters(A_handle, 2, &vertA, &horiz);
+		starpu_data_map_filters(B_handle, 2, &vert, &horizB);
+		starpu_data_map_filters(C_handle, 2, &vert, &horiz);
+
+		for (y = 0; y < nslicesy; y++)
+		for (z = 0; z < nslicesz; z++)
+			starpu_data_set_coordinates(starpu_data_get_sub_data(A_handle, 2, z, y), 2, z, y);
+
+		for (x = 0; x < nslicesx; x++)
+		for (z = 0; z < nslicesz; z++)
+			starpu_data_set_coordinates(starpu_data_get_sub_data(B_handle, 2, x, z), 2, x, z);
+	}
+	else
+	{
+		starpu_data_partition(B_handle, &vert);
+		starpu_data_partition(A_handle, &horiz);
+
+		starpu_data_map_filters(C_handle, 2, &vert, &horiz);
+
+		for (y = 0; y < nslicesy; y++)
+			starpu_data_set_coordinates(starpu_data_get_sub_data(A_handle, 1, y), 2, 0, y);
+
+		for (x = 0; x < nslicesx; x++)
+			starpu_data_set_coordinates(starpu_data_get_sub_data(B_handle, 1, x), 2, x, 0);
+	}
+
+	for (x = 0; x < nslicesx; x++)
+	for (y = 0; y < nslicesy; y++)
+		starpu_data_set_coordinates(starpu_data_get_sub_data(C_handle, 2, x, y), 2, x, y);
 }
 
 #ifdef STARPU_USE_CUDA
-static void cublas_mult(void *descr[], void *arg)
+static void cublas_mult(void *descr[], void *arg, const TYPE *beta)
 {
 	(void)arg;
 	TYPE *subA = (TYPE *)STARPU_MATRIX_GET_PTR(descr[0]);
@@ -175,14 +232,60 @@ static void cublas_mult(void *descr[], void *arg)
 			CUBLAS_OP_N, CUBLAS_OP_N,
 			nxC, nyC, nyA,
 			&p1, subA, ldA, subB, ldB,
-			&v0, subC, ldC);
+			beta, subC, ldC);
 	if (status != CUBLAS_STATUS_SUCCESS)
 		STARPU_CUBLAS_REPORT_ERROR(status);
 }
+
+static void cublas_gemm0(void *descr[], void *arg)
+{
+	cublas_mult(descr, arg, &v0);
+}
+
+static void cublas_gemm(void *descr[], void *arg)
+{
+	cublas_mult(descr, arg, &p1);
+}
+#endif
+
+#ifdef STARPU_USE_HIP
+static void hipblas_mult(void *descr[], void *arg, const TYPE *beta)
+{
+        (void)arg;
+        TYPE *subA = (TYPE *)STARPU_MATRIX_GET_PTR(descr[0]);
+        TYPE *subB = (TYPE *)STARPU_MATRIX_GET_PTR(descr[1]);
+        TYPE *subC = (TYPE *)STARPU_MATRIX_GET_PTR(descr[2]);
+
+        unsigned nxC = STARPU_MATRIX_GET_NX(descr[2]);
+        unsigned nyC = STARPU_MATRIX_GET_NY(descr[2]);
+        unsigned nyA = STARPU_MATRIX_GET_NY(descr[0]);
+
+        unsigned ldA = STARPU_MATRIX_GET_LD(descr[0]);
+        unsigned ldB = STARPU_MATRIX_GET_LD(descr[1]);
+        unsigned ldC = STARPU_MATRIX_GET_LD(descr[2]);
+
+        hipblasStatus_t status = HIPBLAS_GEMM(starpu_hipblas_get_local_handle(),
+                        HIPBLAS_OP_N, HIPBLAS_OP_N,
+                        nxC, nyC, nyA,
+                        &p1, subA, ldA, subB, ldB,
+                        beta, subC, ldC);
+        if (status != HIPBLAS_STATUS_SUCCESS)
+                STARPU_HIPBLAS_REPORT_ERROR(status);
+}
+
+static void hipblas_gemm0(void *descr[], void *arg)
+{
+        hipblas_mult(descr, arg, &v0);
+}
+
+static void hipblas_gemm(void *descr[], void *arg)
+{
+        hipblas_mult(descr, arg, &p1);
+}
 #endif
 
 #ifdef STARPU_HAVE_BLAS
-void cpu_mult(void *descr[], void *arg)
+void cpu_mult(void *descr[], void *arg, TYPE beta)
 {
 	(void)arg;
 	TYPE *subA = (TYPE *)STARPU_MATRIX_GET_PTR(descr[0]);
@@ -202,7 +305,7 @@ void cpu_mult(void *descr[], void *arg)
 	if (worker_size == 1)
 	{
 		/* Sequential CPU task */
-		CPU_GEMM("N", "N", nxC, nyC, nyA, (TYPE)1.0, subA, ldA, subB, ldB, (TYPE)0.0, subC, ldC);
+		CPU_GEMM("N", "N", nxC, nyC, nyA, (TYPE)1.0, subA, ldA, subB, ldB, beta, subC, ldC);
 	}
 	else
 	{
@@ -217,9 +320,19 @@ void cpu_mult(void *descr[], void *arg)
 		TYPE *new_subB = &subB[block_size*rank];
 		TYPE *new_subC = &subC[block_size*rank];
 
-		CPU_GEMM("N", "N", nxC, new_nyC, nyA, (TYPE)1.0, subA, ldA, new_subB, ldB, (TYPE)0.0, new_subC, ldC);
+		CPU_GEMM("N", "N", nxC, new_nyC, nyA, (TYPE)1.0, subA, ldA, new_subB, ldB, beta, new_subC, ldC);
 	}
 }
+
+void cpu_gemm0(void *descr[], void *arg)
+{
+	cpu_mult(descr, arg, 0.);
+}
+
+void cpu_gemm(void *descr[], void *arg)
+{
+	cpu_mult(descr, arg, 1.);
+}
 #endif
 
 static struct starpu_perfmodel starpu_gemm_model =
@@ -228,20 +341,45 @@ static struct starpu_perfmodel starpu_gemm_model =
 	.symbol = STARPU_GEMM_STR(gemm)
 };
 
-static struct starpu_codelet cl =
+static struct starpu_codelet cl_gemm0 =
 {
 #ifdef STARPU_HAVE_BLAS
 	.type = STARPU_SEQ, /* changed to STARPU_SPMD if -spmd is passed */
 	.max_parallelism = INT_MAX,
-	.cpu_funcs = {cpu_mult},
-	.cpu_funcs_name = {"cpu_mult"},
+	.cpu_funcs = {cpu_gemm0},
+	.cpu_funcs_name = {"cpu_gemm0"},
 #endif
 #ifdef STARPU_USE_CUDA
-	.cuda_funcs = {cublas_mult},
+	.cuda_funcs = {cublas_gemm0},
+#elif defined(STARPU_USE_HIP)
+	.hip_funcs = {hipblas_gemm0},
 #elif defined(STARPU_SIMGRID)
 	.cuda_funcs = {(void*)1},
 #endif
 	.cuda_flags = {STARPU_CUDA_ASYNC},
+	.hip_flags = {STARPU_HIP_ASYNC},
+	.nbuffers = 3,
+	.modes = {STARPU_R, STARPU_R, STARPU_W},
+	.model = &starpu_gemm_model
+};
+
+static struct starpu_codelet cl_gemm =
+{
+#ifdef STARPU_HAVE_BLAS
+	.type = STARPU_SEQ, /* changed to STARPU_SPMD if -spmd is passed */
+	.max_parallelism = INT_MAX,
+	.cpu_funcs = {cpu_gemm},
+	.cpu_funcs_name = {"cpu_gemm"},
+#endif
+#ifdef STARPU_USE_CUDA
+	.cuda_funcs = {cublas_gemm},
+#elif defined(STARPU_USE_HIP)
+	.hip_funcs = {hipblas_gemm},
+#elif defined(STARPU_SIMGRID)
+	.cuda_funcs = {(void*)1},
+#endif
+	.cuda_flags = {STARPU_CUDA_ASYNC},
+	.hip_flags = {STARPU_HIP_ASYNC},
 	.nbuffers = 3,
 	.modes = {STARPU_R, STARPU_R, STARPU_RW},
 	.model = &starpu_gemm_model
@@ -252,21 +390,22 @@ static void parse_args(int argc, char **argv)
 	int i;
 	for (i = 1; i < argc; i++)
 	{
-		if (strcmp(argv[i], "-nblocks") == 0)
+		if (strcmp(argv[i], "-3d") == 0)
+		{
+			tiled = 1;
+		}
+
+		else if (strcmp(argv[i], "-nblocks") == 0)
 		{
 			char *argptr;
 			nslicesx = strtol(argv[++i], &argptr, 10);
 			nslicesy = nslicesx;
+			nslicesz = nslicesx;
 			if (nslicesx == 0)
 			{
 				fprintf(stderr, "the number of blocks in X cannot be 0!\n");
 				exit(EXIT_FAILURE);
 			}
-			if (nslicesy == 0)
-			{
-				fprintf(stderr, "the number of blocks in Y cannot be 0!\n");
-				exit(EXIT_FAILURE);
-			}
 		}
 
 		else if (strcmp(argv[i], "-nblocksx") == 0)
@@ -291,40 +430,99 @@ static void parse_args(int argc, char **argv)
 			}
 		}
 
+		else if (strcmp(argv[i], "-nblocksz") == 0)
+		{
+			char *argptr;
+			nslicesz = strtol(argv[++i], &argptr, 10);
+			if (nslicesz == 0)
+			{
+				fprintf(stderr, "the number of blocks in Z cannot be 0!\n");
+				exit(EXIT_FAILURE);
+			}
+		}
+
 		else if (strcmp(argv[i], "-x") == 0)
 		{
 			char *argptr;
 			xdim = strtol(argv[++i], &argptr, 10);
+			if (xdim == 0)
+			{
+				fprintf(stderr, "the X dimension cannot be 0!\n");
+				exit(EXIT_FAILURE);
+			}
+			size_set = 1;
 		}
 
 		else if (strcmp(argv[i], "-xy") == 0)
 		{
 			char *argptr;
 			xdim = ydim = strtol(argv[++i], &argptr, 10);
+			if (xdim == 0)
+			{
+				fprintf(stderr, "the XY dimensions cannot be 0!\n");
+				exit(EXIT_FAILURE);
+			}
+			size_set = 1;
+		}
+
+		else if (strcmp(argv[i], "-xyz") == 0)
+		{
+			char *argptr;
+			xdim = ydim = zdim = strtol(argv[++i], &argptr, 10);
+			size_set = 1;
 		}
 
 		else if (strcmp(argv[i], "-y") == 0)
 		{
 			char *argptr;
 			ydim = strtol(argv[++i], &argptr, 10);
+			if (ydim == 0)
+			{
+				fprintf(stderr, "the Y dimension cannot be 0!\n");
+				exit(EXIT_FAILURE);
+			}
+			size_set = 1;
 		}
 
 		else if (strcmp(argv[i], "-z") == 0)
 		{
 			char *argptr;
 			zdim = strtol(argv[++i], &argptr, 10);
+			if (zdim == 0)
+			{
+				fprintf(stderr, "the Z dimension cannot be 0!\n");
+				exit(EXIT_FAILURE);
+			}
+			size_set = 1;
 		}
 
 		else if (strcmp(argv[i], "-size") == 0)
 		{
 			char *argptr;
 			xdim = ydim = zdim = strtol(argv[++i], &argptr, 10);
+			if (xdim == 0)
+			{
+				fprintf(stderr, "the size cannot be 0!\n");
+				exit(EXIT_FAILURE);
+			}
+			size_set = 1;
 		}
 
 		else if (strcmp(argv[i], "-iter") == 0)
 		{
 			char *argptr;
 			niter = strtol(argv[++i], &argptr, 10);
+			if (niter == 0)
+			{
+				fprintf(stderr, "the number of iterations cannot be 0!\n");
+				exit(EXIT_FAILURE);
+			}
+		}
+
+		else if (strcmp(argv[i], "-nsleeps") == 0)
+		{
+			char *argptr;
+			nsleeps = strtol(argv[++i], &argptr, 10);
 		}
 
 		else if (strcmp(argv[i], "-bound") == 0)
@@ -332,6 +530,11 @@ static void parse_args(int argc, char **argv)
 			bound = 1;
 		}
 
+		else if (strcmp(argv[i], "-hostname") == 0)
+		{
+			print_hostname = 1;
+		}
+
 		else if (strcmp(argv[i], "-check") == 0)
 		{
 			check = 1;
@@ -339,13 +542,16 @@ static void parse_args(int argc, char **argv)
 
 		else if (strcmp(argv[i], "-spmd") == 0)
 		{
-			cl.type = STARPU_SPMD;
+			cl_gemm0.type = STARPU_SPMD;
 		}
 
 		else if (strcmp(argv[i], "-help") == 0 || strcmp(argv[i], "--help") == 0 || strcmp(argv[i], "-h") == 0)
 		{
-			fprintf(stderr,"Usage: %s [-nblocks n] [-nblocksx x] [-nblocksy y] [-x x] [-y y] [-xy n] [-z z] [-size size] [-iter iter] [-bound] [-check] [-spmd]\n", argv[0]);
-			fprintf(stderr,"Currently selected: %ux%u * %ux%u and %ux%u blocks (size %ux%u length %u), %u iterations\n", zdim, ydim, xdim, zdim, nslicesx, nslicesy, xdim / nslicesx, ydim / nslicesy, zdim, niter);
+			fprintf(stderr,"Usage: %s [-3d] [-nblocks n] [-nblocksx x] [-nblocksy y] [-nblocksz z] [-x x] [-y y] [-xy n] [-z z] [-xyz n] [-size size] [-iter iter] [-bound] [-check] [-spmd] [-hostname] [-nsleeps nsleeps]\n", argv[0]);
+			if (tiled)
+				fprintf(stderr,"Currently selected: %ux%u * %ux%u and %ux%ux%u blocks (size %ux%u length %u), %u iterations, %u sleeps\n", zdim, ydim, xdim, zdim, nslicesx, nslicesy, nslicesz, xdim / nslicesx, ydim / nslicesy, zdim / nslicesz, niter, nsleeps);
+			else
+				fprintf(stderr,"Currently selected: %ux%u * %ux%u and %ux%u blocks (size %ux%u length %u), %u iterations, %u sleeps\n", zdim, ydim, xdim, zdim, nslicesx, nslicesy, xdim / nslicesx, ydim / nslicesy, zdim, niter, nsleeps);
 			exit(EXIT_SUCCESS);
 		}
 		else
@@ -363,8 +569,19 @@ int main(int argc, char **argv)
 
 	parse_args(argc, argv);
 
+#ifndef STARPU_SIMGRID
+	if (check && !size_set)
+	{
+		/* Check is sequential, reduce its default duration */
+		xdim /= 2;
+		ydim /= 2;
+	}
+#endif
+
 #ifdef STARPU_QUICK_CHECK
 	niter /= 10;
+	if(niter==0)
+		niter=1;
 #endif
 
 	starpu_fxt_autostart_profiling(0);
@@ -374,70 +591,125 @@ int main(int argc, char **argv)
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
 	starpu_cublas_init();
+	starpu_hipblas_init();
 
 	init_problem_data();
 	partition_mult_data();
 
+	PRINTF("# ");
+	if (print_hostname)
+		PRINTF("node\t");
+	PRINTF("x\ty\tz\tms\tGFlop/s");
 	if (bound)
-		starpu_bound_start(0, 0);
+		PRINTF("\tTms\tTGFlop/s\tTims\tTiGFlop/s");
+	PRINTF("\n");
 
-	starpu_fxt_start_profiling();
-	start = starpu_timing_now();
-
-	unsigned x, y, iter;
-	for (iter = 0; iter < niter; iter++)
+	unsigned sleeps;
+	for (sleeps = 0; sleeps < nsleeps; sleeps++)
 	{
-		for (x = 0; x < nslicesx; x++)
-		for (y = 0; y < nslicesy; y++)
+		if (bound)
+			starpu_bound_start(0, 0);
+
+	        starpu_fxt_start_profiling();
+                start = starpu_timing_now();
+
+		unsigned x, y, z, iter;
+		for (iter = 0; iter < niter; iter++)
 		{
-			struct starpu_task *task = starpu_task_create();
-
-			task->cl = &cl;
-
-			task->handles[0] = starpu_data_get_sub_data(A_handle, 1, y);
-			task->handles[1] = starpu_data_get_sub_data(B_handle, 1, x);
-			task->handles[2] = starpu_data_get_sub_data(C_handle, 2, x, y);
-
-			task->flops = 2ULL * (xdim/nslicesx) * (ydim/nslicesy) * zdim;
-
-			ret = starpu_task_submit(task);
-			if (ret == -ENODEV)
+			if (tiled)
 			{
-			     check = 0;
-			     ret = 77;
-			     goto enodev;
+				for (x = 0; x < nslicesx; x++)
+				for (y = 0; y < nslicesy; y++)
+				{
+					starpu_data_handle_t Ctile = starpu_data_get_sub_data(C_handle, 2, x, y);
+					for (z = 0; z < nslicesz; z++)
+					{
+						struct starpu_task *task = starpu_task_create();
+
+						if (z == 0)
+							task->cl = &cl_gemm0;
+						else
+							task->cl = &cl_gemm;
+
+						task->handles[0] = starpu_data_get_sub_data(A_handle, 2, z, y);
+						task->handles[1] = starpu_data_get_sub_data(B_handle, 2, x, z);
+						task->handles[2] = Ctile;
+
+						task->flops = 2ULL * (xdim/nslicesx) * (ydim/nslicesy) * (zdim/nslicesz);
+
+						ret = starpu_task_submit(task);
+						if (ret == -ENODEV)
+						{
+						     check = 0;
+						     ret = 77;
+						     goto enodev;
+						}
+						STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
+					}
+					starpu_data_wont_use(Ctile);
+				}
 			}
-			STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
-			starpu_data_wont_use(starpu_data_get_sub_data(C_handle, 2, x, y));
+			else
+			{
+				for (x = 0; x < nslicesx; x++)
+				for (y = 0; y < nslicesy; y++)
+				{
+					struct starpu_task *task = starpu_task_create();
+
+					task->cl = &cl_gemm0;
+
+					task->handles[0] = starpu_data_get_sub_data(A_handle, 1, y);
+					task->handles[1] = starpu_data_get_sub_data(B_handle, 1, x);
+					task->handles[2] = starpu_data_get_sub_data(C_handle, 2, x, y);
+
+					task->flops = 2ULL * (xdim/nslicesx) * (ydim/nslicesy) * zdim;
+
+					ret = starpu_task_submit(task);
+					if (ret == -ENODEV)
+					{
+					     check = 0;
+					     ret = 77;
+					     goto enodev;
+					}
+					STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
+					starpu_data_wont_use(starpu_data_get_sub_data(C_handle, 2, x, y));
+				}
+			}
+
+			starpu_task_wait_for_all();
 		}
 
-		starpu_task_wait_for_all();
+		end = starpu_timing_now();
+		starpu_fxt_stop_profiling();
+
+		if (bound)
+			starpu_bound_stop();
+
+		double timing = end - start;
+		double min, min_int;
+		double flops = 2.0*((unsigned long long)(niter))*((unsigned long long)xdim)
+				   *((unsigned long long)ydim)*((unsigned long long)zdim);
+
+		if (bound)
+			starpu_bound_compute(&min, &min_int, 1);
+
+		if (print_hostname)
+		{
+			char hostname[255];
+			gethostname(hostname, 255);
+			PRINTF("%s\t", hostname);
+		}
+		PRINTF("%u\t%u\t%u\t%.0f\t%.1f", xdim, ydim, zdim, timing/(niter)/1000.0, flops/timing/1000.0);
+		if (bound)
+			PRINTF("\t%.0f\t%.1f\t%.0f\t%.1f", min, flops/min/1000000.0, min_int, flops/min_int/1000000.0);
+		PRINTF("\n");
+
+		if (sleeps < nsleeps-1)
+		{
+			sleep(10);
+		}
 	}
 
-
-	end = starpu_timing_now();
-	starpu_fxt_stop_profiling();
-
-	if (bound)
-		starpu_bound_stop();
-
-	double timing = end - start;
-	double min, min_int;
-	double flops = 2.0*((unsigned long long)niter)*((unsigned long long)xdim)
-		           *((unsigned long long)ydim)*((unsigned long long)zdim);
-
-	if (bound)
-		starpu_bound_compute(&min, &min_int, 1);
-
-	PRINTF("# x\ty\tz\tms\tGFlops");
-	if (bound)
-		PRINTF("\tTms\tTGFlops\tTims\tTiGFlops");
-	PRINTF("\n");
-	PRINTF("%u\t%u\t%u\t%.0f\t%.1f", xdim, ydim, zdim, timing/niter/1000.0, flops/timing/1000.0);
-	if (bound)
-		PRINTF("\t%.0f\t%.1f\t%.0f\t%.1f", min, flops/min/1000000.0, min_int, flops/min_int/1000000.0);
-	PRINTF("\n");
-
 enodev:
 	starpu_data_unpartition(C_handle, STARPU_MAIN_RAM);
 	starpu_data_unpartition(B_handle, STARPU_MAIN_RAM);
@@ -459,6 +731,7 @@ enodev:
 	starpu_free_flags(C, xdim*ydim*sizeof(TYPE), STARPU_MALLOC_PINNED|STARPU_MALLOC_SIMULATION_FOLDED);
 
 	starpu_cublas_shutdown();
+	starpu_hipblas_shutdown();
 	starpu_shutdown();
 
 	return ret;
