@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -82,7 +82,6 @@ int main(void)
 {
 	int ntasks = NTASKS;
 	int ret;
-	unsigned ncuda = 0;
 	int nprocs1 = 0;
 	int nprocs2 = 0;
 	int procs1[STARPU_NMAXWORKERS], procs2[STARPU_NMAXWORKERS];
@@ -93,8 +92,8 @@ int main(void)
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
 #ifdef STARPU_HAVE_VALGRIND_H
-       if (RUNNING_ON_VALGRIND)
-               ntasks = 8;
+	if (RUNNING_ON_VALGRIND)
+		ntasks = 8;
 #endif
 
 #ifdef STARPU_USE_CPU
@@ -105,7 +104,7 @@ int main(void)
 	if (nprocs1 == 0) goto enodev;
 
 #ifdef STARPU_USE_CUDA
-	ncuda = nprocs2 = starpu_cuda_worker_get_count();
+	nprocs2 = starpu_cuda_worker_get_count();
 	starpu_worker_get_ids_by_type(STARPU_CUDA_WORKER, procs2, nprocs2);
 #endif
 	if (nprocs2 == 0)
