@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2014-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2014-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -21,7 +21,7 @@
 
 /**
    @defgroup API_OpenMP_Runtime_Support OpenMP Runtime Support
-   @brief This section describes the interface provided for implementing OpenMP runtimes on top of StarPU.
+   @brief API for implementing OpenMP runtimes on top of StarPU.
    @{
 */
 
@@ -70,7 +70,7 @@ enum starpu_omp_sched_value
 	starpu_omp_sched_static	   = 1, /**< \b Static iteration scheduling algorithm.*/
 	starpu_omp_sched_dynamic   = 2, /**< \b Dynamic iteration scheduling algorithm.*/
 	starpu_omp_sched_guided	   = 3, /**< \b Guided iteration scheduling algorithm.*/
-	starpu_omp_sched_auto	   = 4, /**< \b Automatically choosen iteration scheduling algorithm.*/
+	starpu_omp_sched_auto	   = 4, /**< \b Automatically chosen iteration scheduling algorithm.*/
 	starpu_omp_sched_runtime   = 5	/**< Choice of iteration scheduling algorithm deferred at \b runtime.*/
 };
 
@@ -402,8 +402,8 @@ extern void *starpu_omp_single_copyprivate_inline_begin(void *data) __STARPU_OMP
    the task that won the election. This function can be used to implement
    <c>\#pragma omp single</c> with a copyprivate clause without code outlining.
 
-   \return the copyprivate pointer for tasks that lost the election and therefore did not execute the code of the single section.
-   \return <c>NULL</c> for the task that won the election and executed the code of the single section.
+   Return the copyprivate pointer for tasks that lost the election and therefore did not execute the code of the single section.
+   Return <c>NULL</c> for the task that won the election and executed the code of the single section.
 
    \sa starpu_omp_single_copyprivate_inline
    \sa starpu_omp_single_copyprivate_inline_begin
@@ -817,7 +817,7 @@ extern int starpu_omp_get_cancellation(void) __STARPU_OMP_NOTHROW;
 /**
    Set the default scheduling kind for upcoming loops within the
    current parallel section. \p kind is the scheduler kind, \p modifier
-   complements the scheduler kind with informations such as the chunk size,
+   complements the scheduler kind with information such as the chunk size,
    in accordance with the OpenMP specification.
 
    \sa starpu_omp_get_schedule
@@ -827,9 +827,7 @@ extern int starpu_omp_get_cancellation(void) __STARPU_OMP_NOTHROW;
 extern void starpu_omp_set_schedule(enum starpu_omp_sched_value kind, int modifier) __STARPU_OMP_NOTHROW;
 
 /**
-   Return the current selected default loop scheduler.
-
-   \return the kind and the modifier of the current default loop scheduler.
+   Return the kind and the modifier of the current default loop scheduler.
 
    \sa starpu_omp_set_schedule
 
@@ -964,8 +962,6 @@ extern int starpu_omp_get_place_num_procs(int place_num) __STARPU_OMP_NOTHROW;
 /**
    Return the numerical identifiers of the processors available to the execution environment in the specified place.
 
-   \return the numerical identifiers of the processors available to the execution environment in the specified place.
-
    See \ref OMPStandard for more details.
 */
 extern void starpu_omp_get_place_proc_ids(int place_num, int *ids) __STARPU_OMP_NOTHROW;
@@ -991,8 +987,6 @@ extern int starpu_omp_get_partition_num_places(void) __STARPU_OMP_NOTHROW;
 /**
    Return the list of place numbers corresponding to the places in the place-partition-var ICV of the innermost implicit task.
 
-   \return the list of place numbers corresponding to the places in the place-partition-var ICV of the innermost implicit task.
-
    See \ref OMPStandard for more details.
 */
 extern void starpu_omp_get_partition_place_nums(int *place_nums) __STARPU_OMP_NOTHROW;
