@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2011-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,12 +16,7 @@
 
 #include <starpu_mpi.h>
 #include <math.h>
-
-#define FPRINTF(ofile, fmt, ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ## __VA_ARGS__); }} while(0)
-#define FPRINTF_MPI(ofile, fmt, ...) do { if (!getenv("STARPU_SSILENT")) { \
-    						int _disp_rank; starpu_mpi_comm_rank(MPI_COMM_WORLD, &_disp_rank);       \
-                                                fprintf(ofile, "[%d][starpu_mpi][%s] " fmt , _disp_rank, __starpu_func__ ,## __VA_ARGS__); \
-                                                fflush(ofile); }} while(0);
+#include "helper.h"
 
 void stencil5_cpu(void *descr[], void *_args)
 {
@@ -47,16 +42,16 @@ struct starpu_codelet stencil5_cl =
 
 #ifdef STARPU_QUICK_CHECK
 #  define NITER_DEF	5
-#  define X         	4
-#  define Y         	4
+#  define X		4
+#  define Y		4
 #elif !defined(STARPU_LONG_CHECK)
 #  define NITER_DEF	10
-#  define X         	5
-#  define Y         	5
+#  define X		5
+#  define Y		5
 #else
 #  define NITER_DEF	100
-#  define X         	20
-#  define Y         	20
+#  define X		20
+#  define Y		20
 #endif
 
 int display = 0;
