@@ -1,6 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2021       Federal University of Rio Grande do Sul (UFRGS)
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -22,11 +23,11 @@
 #if defined(STARPU_USE_MPI)
 
 #include <mpi.h>
+#include <starpu_mpi_ft.h>
 #include <stdint.h>
 
 #ifdef __cplusplus
-extern "C"
-{
+extern "C" {
 #endif
 
 /**
@@ -78,30 +79,37 @@ int starpu_mpi_initialize(void) STARPU_DEPRECATED;
 */
 int starpu_mpi_initialize_extended(int *rank, int *world_size) STARPU_DEPRECATED;
 
+/**
+   Call starpu_mpi_shutdown_comm() with the MPI communicator \c MPI_COMM_WORLD
+*/
+int starpu_mpi_shutdown(void);
+
 /**
    Clean the starpumpi library. This must be called after calling any
    \c starpu_mpi functions and before the call to starpu_shutdown(),
    if any. \c MPI_Finalize() will be called if StarPU-MPI has been
    initialized by starpu_mpi_init().
 */
-int starpu_mpi_shutdown(void);
+int starpu_mpi_shutdown_comm(MPI_Comm comm);
 
 /**
-   Retrieve the current amount of communications from the current node
-   in the array \p comm_amounts which must have a size greater or
-   equal to the world size. Communications statistics must be enabled
-   (see \ref STARPU_COMM_STATS).
+   Register \p comm. The function is automatically called for the
+   communicator given to starpu_mpi_init_comm().
 */
-void starpu_mpi_comm_amounts_retrieve(size_t *comm_amounts);
+int starpu_mpi_comm_register(MPI_Comm comm);
 
 /**
-   Return in \p size the size of the communicator \p comm
+   Return in \p size the size of the communicator \p comm. The
+   function will fail if starpu_mpi_comm_register() has not been
+   previously called with the given communicator.
 */
 int starpu_mpi_comm_size(MPI_Comm comm, int *size);
 
 /**
    Return in \p rank the rank of the calling process in the
-   communicator \p comm
+   communicator \p comm.  The function will fail if
+   starpu_mpi_comm_register() has not been previously called with the
+   given communicator.
 */
 int starpu_mpi_comm_rank(MPI_Comm comm, int *rank);
 
@@ -132,7 +140,19 @@ int starpu_mpi_world_size(void);
 */
 int starpu_mpi_comm_get_attr(MPI_Comm comm, int keyval, void *attribute_val, int *flag);
 
+/**
+   Get the logical index of the core where the MPI thread is bound.
+*/
+int starpu_mpi_get_thread_cpuid(void);
+
+/**
+   Get the tag used for MPI communications submitted by StarPU.
+*/
 int starpu_mpi_get_communication_tag(void);
+
+/**
+   Set the tag used for MPI communications submitted by StarPU.
+*/
 void starpu_mpi_set_communication_tag(int tag);
 
 /** @} */
@@ -191,9 +211,16 @@ int starpu_mpi_send_prio(starpu_data_handle_t data_handle, int dest, starpu_mpi_
    Perform a standard-mode, blocking receive in \p data_handle from
    the node \p source using the message tag \p data_tag within the
    communicator \p comm.
+   The value of \p status cannot be NULL, use the predefined value
+   MPI_STATUS_IGNORE to ignore the status.
 */
 int starpu_mpi_recv(starpu_data_handle_t data_handle, int source, starpu_mpi_tag_t data_tag, MPI_Comm comm, MPI_Status *status);
 
+/**
+   Similar to starpu_mpi_recv(), but take a priority \p prio
+*/
+int starpu_mpi_recv_prio(starpu_data_handle_t data_handle, int source, starpu_mpi_tag_t data_tag, int prio, MPI_Comm comm, MPI_Status *status);
+
 /**
    Post a standard-mode, non blocking send of \p data_handle to the
    node \p dest using the message tag \p data_tag within the
@@ -207,7 +234,7 @@ int starpu_mpi_recv(starpu_data_handle_t data_handle, int source, starpu_mpi_tag
 int starpu_mpi_isend_detached(starpu_data_handle_t data_handle, int dest, starpu_mpi_tag_t data_tag, MPI_Comm comm, void (*callback)(void *), void *arg);
 
 /**
-   Similar to starpu_mpi_isend_detached, but take a priority \p prio.
+   Similar to starpu_mpi_isend_detached(), but take a priority \p prio.
 */
 int starpu_mpi_isend_detached_prio(starpu_data_handle_t data_handle, int dest, starpu_mpi_tag_t data_tag, int prio, MPI_Comm comm, void (*callback)(void *), void *arg);
 
@@ -223,6 +250,11 @@ int starpu_mpi_isend_detached_prio(starpu_data_handle_t data_handle, int dest, s
 */
 int starpu_mpi_irecv_detached(starpu_data_handle_t data_handle, int source, starpu_mpi_tag_t data_tag, MPI_Comm comm, void (*callback)(void *), void *arg);
 
+/**
+   Same of starpu_mpi_irecv_detached but with the \p prio parameter.
+*/
+int starpu_mpi_irecv_detached_prio(starpu_data_handle_t data_handle, int source, starpu_mpi_tag_t data_tag, int prio, MPI_Comm comm, void (*callback)(void *), void *arg);
+
 /**
    Post a nonblocking receive in \p data_handle from the node \p
    source using the message tag \p data_tag within the communicator \p
@@ -271,6 +303,8 @@ int starpu_mpi_issend_detached_prio(starpu_data_handle_t data_handle, int dest,
 
 /**
    Return when the operation identified by request \p req is complete.
+   The value of \p status cannot be NULL, use the predefined value
+   MPI_STATUS_IGNORE to ignore the status.
 */
 int starpu_mpi_wait(starpu_mpi_req *req, MPI_Status *status);
 
@@ -338,6 +372,7 @@ int starpu_mpi_isend_array_detached_unlock_tag_prio(unsigned array_size, starpu_
 int starpu_mpi_irecv_array_detached_unlock_tag(unsigned array_size, starpu_data_handle_t *data_handle, int *source, starpu_mpi_tag_t *data_tag, MPI_Comm *comm, starpu_tag_t tag);
 
 typedef int (*starpu_mpi_datatype_allocate_func_t)(starpu_data_handle_t, MPI_Datatype *);
+typedef int (*starpu_mpi_datatype_node_allocate_func_t)(starpu_data_handle_t, unsigned node, MPI_Datatype *);
 typedef void (*starpu_mpi_datatype_free_func_t)(MPI_Datatype *);
 
 /**
@@ -360,6 +395,26 @@ int starpu_mpi_datatype_register(starpu_data_handle_t handle, starpu_mpi_datatyp
 */
 int starpu_mpi_interface_datatype_register(enum starpu_data_interface_id id, starpu_mpi_datatype_allocate_func_t allocate_datatype_func, starpu_mpi_datatype_free_func_t free_datatype_func);
 
+/**
+   Register functions to create and free a MPI datatype for the given
+   handle.
+   Similar to starpu_mpi_interface_datatype_register().
+   It is important that the function is called before any
+   communication can take place for a data with the given handle. See
+   \ref ExchangingUserDefinedDataInterface for an example.
+*/
+int starpu_mpi_datatype_node_register(starpu_data_handle_t handle, starpu_mpi_datatype_node_allocate_func_t allocate_datatype_func, starpu_mpi_datatype_free_func_t free_datatype_func);
+
+/**
+   Register functions to create and free a MPI datatype for the given
+   interface id.
+   Similar to starpu_mpi_datatype_register().
+   It is important that the function is called before any
+   communication can take place for a data with the given handle. See
+   \ref ExchangingUserDefinedDataInterface for an example.
+*/
+int starpu_mpi_interface_datatype_node_register(enum starpu_data_interface_id id, starpu_mpi_datatype_node_allocate_func_t allocate_datatype_func, starpu_mpi_datatype_free_func_t free_datatype_func);
+
 /**
    Unregister the MPI datatype functions stored for the interface of
    the given handle.
@@ -382,7 +437,7 @@ int starpu_mpi_interface_datatype_unregister(enum starpu_data_interface_id id);
 /**
    Return 1 if the communication cache is enabled, 0 otherwise
 */
-int starpu_mpi_cache_is_enabled();
+int starpu_mpi_cache_is_enabled(void);
 
 /**
    If \p enabled is 1, enable the communication cache. Otherwise,
@@ -422,6 +477,7 @@ int starpu_mpi_cached_receive(starpu_data_handle_t data_handle);
  * Return 0 if the communication cache is not enabled
  */
 int starpu_mpi_cached_receive_set(starpu_data_handle_t data);
+int starpu_mpi_cached_cp_receive_set(starpu_data_handle_t data_handle);
 
 /**
  * Remove \p data from the reception cache
@@ -530,6 +586,10 @@ int starpu_mpi_data_get_rank(starpu_data_handle_t handle);
    Return the tag of the given data.
 */
 starpu_mpi_tag_t starpu_mpi_data_get_tag(starpu_data_handle_t handle);
+/**
+   Return the redux map of the given data.
+*/
+char *starpu_mpi_data_get_redux_map(starpu_data_handle_t handle);
 
 /**
    Symbol kept for backward compatibility. Call function starpu_mpi_data_get_tag()
@@ -565,7 +625,7 @@ starpu_mpi_tag_t starpu_mpi_data_get_tag(starpu_data_handle_t handle);
 	starpu_mpi_node_selection_set_current_policy())
 	<li>
 	The argument ::STARPU_EXECUTE_ON_NODE followed by an integer
-	can be used to specify the node;
+	can be used to specify the node; Ignored if the node value is -1.
 	<li>
 	The argument ::STARPU_EXECUTE_ON_DATA followed by a data handle can be used to specify that the node owing the given data will execute the codelet.
 	</ul>
@@ -591,11 +651,19 @@ starpu_mpi_tag_t starpu_mpi_data_get_tag(starpu_data_handle_t handle);
    STARPU_MPI_CACHE).
 */
 int starpu_mpi_task_insert(MPI_Comm comm, struct starpu_codelet *codelet, ...);
+#ifdef STARPU_USE_FXT
+#define starpu_mpi_task_insert(comm, cl, ...) \
+	starpu_mpi_task_insert(comm, cl, STARPU_TASK_FILE, __FILE__, STARPU_TASK_LINE, __LINE__, ##__VA_ARGS__)
+#endif
 
 /**
-   Call starpu_mpi_task_insert(). Symbol kept for backward compatibility.
+   Identical to starpu_mpi_task_insert(). Symbol kept for backward compatibility.
 */
 int starpu_mpi_insert_task(MPI_Comm comm, struct starpu_codelet *codelet, ...);
+#ifdef STARPU_USE_FXT
+#define starpu_mpi_insert_task(comm, cl, ...) \
+	starpu_mpi_insert_task(comm, cl, STARPU_TASK_FILE, __FILE__, STARPU_TASK_LINE, __LINE__, ##__VA_ARGS__)
+#endif
 
 /**
    Create a task corresponding to \p codelet with the following given
@@ -611,6 +679,15 @@ int starpu_mpi_insert_task(MPI_Comm comm, struct starpu_codelet *codelet, ...);
    creates it, with the SAME list of arguments.
 */
 struct starpu_task *starpu_mpi_task_build(MPI_Comm comm, struct starpu_codelet *codelet, ...);
+#ifdef STARPU_USE_FXT
+#define starpu_mpi_task_build(comm, cl, ...) \
+	starpu_mpi_task_build(comm, cl, STARPU_TASK_FILE, __FILE__, STARPU_TASK_LINE, __LINE__, ##__VA_ARGS__)
+#endif
+
+/**
+   Offer a va_list variant of starpu_mpi_task_build.
+ */
+struct starpu_task *starpu_mpi_task_build_v(MPI_Comm comm, struct starpu_codelet *codelet, va_list varg_list);
 
 /**
    MUST be called after a call to starpu_mpi_task_build(),
@@ -619,12 +696,52 @@ struct starpu_task *starpu_mpi_task_build(MPI_Comm comm, struct starpu_codelet *
 */
 int starpu_mpi_task_post_build(MPI_Comm comm, struct starpu_codelet *codelet, ...);
 
+/**
+   Offer a va_list variant of starpu_mpi_task_post_build.
+ */
+int starpu_mpi_task_post_build_v(MPI_Comm comm, struct starpu_codelet *codelet, va_list varg_list);
+
+/**
+   Structure used to pass data from
+   starpu_mpi_task_exchange_data_before_execution() to
+   starpu_mpi_task_exchange_data_after_execution()
+ */
+struct starpu_mpi_task_exchange_params
+{
+	int do_execute; /**< is the caller going to execute the task */
+	int xrank;      /**< node executing the task */
+	int priority;   /**< priority of the task being executed */
+};
+
+/**
+   Perform all necessary communications needed before the execution of
+   the given task. The field \c priority of \p params will be set with
+   the rank of the node which is selected to submit \p task.
+   After calling this function, and the submission of the task for the
+   selected node, all nodes MUST call the function
+   starpu_mpi_task_exchange_data_after_execution() with the parameters
+   \p descrs and \p params.
+ */
+int starpu_mpi_task_exchange_data_before_execution(MPI_Comm comm, struct starpu_task *task, struct starpu_data_descr *descrs, struct starpu_mpi_task_exchange_params *params);
+
+/**
+   MUST be called after a call to
+   starpu_mpi_task_exchange_data_before_execution() with the same
+   arguments \p descrs and \p params.
+   \p nb_data is the number of data in \p descrs.
+   Perform all the necessary communications needed after the execution
+   of the task, i.e the fourth -- last -- step of the algorithm
+   described in starpu_mpi_task_insert().
+*/
+int starpu_mpi_task_exchange_data_after_execution(MPI_Comm comm, struct starpu_data_descr *descrs, unsigned nb_data, struct starpu_mpi_task_exchange_params params);
+
 /**
    Transfer data \p data_handle to MPI node \p node, sending it from
    its owner if needed. At least the target node and the owner have to
    call the function.
+   This waits for the transfer to be over.
 */
-void starpu_mpi_get_data_on_node(MPI_Comm comm, starpu_data_handle_t data_handle, int node);
+int starpu_mpi_get_data_on_node(MPI_Comm comm, starpu_data_handle_t data_handle, int node);
 
 /**
    Transfer data \p data_handle to MPI node \p node, sending it from
@@ -632,7 +749,7 @@ void starpu_mpi_get_data_on_node(MPI_Comm comm, starpu_data_handle_t data_handle
    call the function. On reception, the \p callback function is called
    with the argument \p arg.
 */
-void starpu_mpi_get_data_on_node_detached(MPI_Comm comm, starpu_data_handle_t data_handle, int node, void (*callback)(void*), void *arg);
+int starpu_mpi_get_data_on_node_detached(MPI_Comm comm, starpu_data_handle_t data_handle, int node, void (*callback)(void *), void *arg);
 
 /**
    Transfer data \p data_handle to all MPI nodes, sending it from its
@@ -660,7 +777,6 @@ void starpu_mpi_data_migrate(MPI_Comm comm, starpu_data_handle_t handle, int new
    @{
 */
 
-
 /**
    Define the current policy
  */
@@ -669,7 +785,7 @@ void starpu_mpi_data_migrate(MPI_Comm comm, starpu_data_handle_t handle, int new
    Define the policy in which the selected node is the one having the
    most data in ::STARPU_R mode
 */
-#define STARPU_MPI_NODE_SELECTION_MOST_R_DATA    0
+#define STARPU_MPI_NODE_SELECTION_MOST_R_DATA 0
 
 typedef int (*starpu_mpi_select_node_policy_func_t)(int me, int nb_nodes, struct starpu_data_descr *descr, int nb_data);
 
@@ -714,13 +830,13 @@ int starpu_mpi_node_selection_unregister_policy(int policy);
    Return the current policy used to select the node which will
    execute the codelet
 */
-int starpu_mpi_node_selection_get_current_policy();
+int starpu_mpi_node_selection_get_current_policy(void);
 
 /**
    Set the current policy used to select the node which will execute
    the codelet. The policy ::STARPU_MPI_NODE_SELECTION_MOST_R_DATA
    selects the node having the most data in ::STARPU_R mode so as to
-   minimize the amount of data to be transfered.
+   minimize the amount of data to be transferred.
 */
 int starpu_mpi_node_selection_set_current_policy(int policy);
 
@@ -736,12 +852,24 @@ int starpu_mpi_node_selection_set_current_policy(int policy);
    Perform a reduction on the given data \p handle. All nodes send the
    data to its owner node which will perform a reduction.
 */
-void starpu_mpi_redux_data(MPI_Comm comm, starpu_data_handle_t data_handle);
+int starpu_mpi_redux_data(MPI_Comm comm, starpu_data_handle_t data_handle);
 
 /**
-   Similar to starpu_mpi_redux_data, but take a priority \p prio.
+   Similar to starpu_mpi_redux_data(), but take a priority \p prio.
 */
-void starpu_mpi_redux_data_prio(MPI_Comm comm, starpu_data_handle_t data_handle, int prio);
+int starpu_mpi_redux_data_prio(MPI_Comm comm, starpu_data_handle_t data_handle, int prio);
+
+/**
+   Perform a reduction on the given data \p handle.
+   Nodes perform the reduction through in a tree-based fashion.
+   The tree use is an \p arity - ary tree.
+*/
+int starpu_mpi_redux_data_tree(MPI_Comm comm, starpu_data_handle_t data_handle, int arity);
+
+/**
+   Similar to starpu_mpi_redux_data_tree(), but take a priority \p prio.
+*/
+int starpu_mpi_redux_data_prio_tree(MPI_Comm comm, starpu_data_handle_t data_handle, int prio, int arity);
 
 /**
    Scatter data among processes of the communicator based on the
@@ -769,8 +897,124 @@ int starpu_mpi_gather_detached(starpu_data_handle_t *data_handles, int count, in
 
 /** @} */
 
+/**
+   @name Dynamic Broadcasts
+   \anchor MPIDynamicBroadcasts
+   @{
+*/
+
+/**
+   Enable or disable coop sends.<br>
+   Used for benchmark, not recommended for production: can cause problems
+   if there are still communications while disabling, or when shutting down
+   StarPU.<br>
+   This function must be called after the initialization of StarPU.
+*/
+void starpu_mpi_coop_sends_set_use(int use_coop_sends);
+
+/**
+   Return whether coop sends are enabled or not.
+*/
+int starpu_mpi_coop_sends_get_use(void);
+
+/**
+   Explicit the number of different sends of the \p data_handle. When the number
+   of sends is reached, a collective operation is triggered. If this function isn't
+   called, StarPU will trigger a collective operation containing only posted sends
+   while the data wasn't available.
+*/
+void starpu_mpi_coop_sends_data_handle_nb_sends(starpu_data_handle_t data_handle, int nb_sends);
+
+/** @} */
+
+/**
+   @name Statistics
+   \anchor MPIStats
+   @{
+*/
+
+/**
+   Disable the aggregation of communications statistics.
+*/
+void starpu_mpi_comm_stats_disable(void);
+
+/**
+   Enable the aggregation of communications statistics.
+*/
+void starpu_mpi_comm_stats_enable(void);
+
+/**
+   Retrieve the current communications statistics from the current node
+   in the array \p comm_stats which must have a size greater or
+   equal to the world size. Communications statistics must have been enabled,
+   either through the function starpu_mpi_comm_stats_enable() or
+   through the environment variable \ref STARPU_MPI_STATS.
+*/
+void starpu_mpi_comm_stats_retrieve(size_t *comm_stats);
+
+/** @} */
+
+/**
+   @name Miscellaneous
+   \anchor MPIMisc
+   @{
+*/
+
 int starpu_mpi_pre_submit_hook_register(void (*f)(struct starpu_task *));
-int starpu_mpi_pre_submit_hook_unregister();
+int starpu_mpi_pre_submit_hook_unregister(void);
+
+/**
+   Copy the content of \p src_handle into \p dst_handle. If both data
+   are on the same node, the function starpu_data_cpy() is called,
+   otherwise a MPI transfer is initiated between both nodes.
+   The parameter \p asynchronous indicates whether the function should
+   block or not.
+   If \p callback_func is not <c>NULL</c>, this callback function
+   is executed on the owner node of the data \p dst_handle after the
+   handle has been received, and it is given the pointer \p
+   callback_arg as argument.
+   See \ref MPITaskUtility for more details.
+*/
+int starpu_mpi_data_cpy(starpu_data_handle_t dst_handle, starpu_data_handle_t src_handle, MPI_Comm comm, int asynchronous, void (*callback_func)(void *), void *callback_arg);
+
+/**
+   Similar to starpu_mpi_data_cpy(), but take a priority \p prio.
+*/
+int starpu_mpi_data_cpy_priority(starpu_data_handle_t dst_handle, starpu_data_handle_t src_handle, MPI_Comm comm, int asynchronous, void (*callback_func)(void *), void *callback_arg, int priority);
+
+/** @} */
+
+/**
+   @name Data Tags Management
+   \anchor MPITags
+   @{
+*/
+
+/**
+   Book a range of unique tags of size \p nbtags to be used to register
+   StarPU data handles.
+   This function returns the minimal tag value available \c mintag to
+   allow the registration of data with tags in the continuous range [[
+   \c mintag, \c mintag + \p nbtags ]]
+
+   Note that this function must be called by all MPI processes
+   involved in the computations with the same parameters and in the
+   exact same order to make sure the tags are identical from one node
+   to another.
+*/
+int64_t starpu_mpi_tags_allocate(int64_t nbtags);
+
+/**
+   Release the range of tags starting by the given \p mintag value. The
+   mintag value must be a value obtained through a call to
+   starpu_mpi_tags_allocate().
+
+   Note that this function must be called by all MPI processes
+   involved in the computations with the same parameters and in the
+   exact same order to make sure the tags are identical from one node
+   to another as for starpu_mpi_tags_allocate().
+*/
+void starpu_mpi_tags_free(int64_t mintag);
 
 /** @} */
 
