@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -48,7 +48,7 @@ int main(int argc, char **argv)
 		starpu_mpi_shutdown();
 		if (!mpi_init)
 			MPI_Finalize();
-		return STARPU_TEST_SKIPPED;
+		return rank == 0 ? STARPU_TEST_SKIPPED : 0;
 	}
 
 	tab = calloc(SIZE, sizeof(float));
@@ -63,14 +63,17 @@ int main(int argc, char **argv)
 	{
 		if ((loop % 2) == (rank%2))
 		{
-			starpu_mpi_send(tab_handle, other_rank, loop, MPI_COMM_WORLD);
+			ret = starpu_mpi_send(tab_handle, other_rank, loop, MPI_COMM_WORLD);
+			STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_send");
 		}
 		else
 		{
 			MPI_Status status;
 			starpu_mpi_req req;
-			starpu_mpi_irecv(tab_handle, &req, other_rank, loop, MPI_COMM_WORLD);
-			starpu_mpi_wait(&req, &status);
+			ret = starpu_mpi_irecv(tab_handle, &req, other_rank, loop, MPI_COMM_WORLD);
+			STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_irecv");
+			ret = starpu_mpi_wait(&req, &status);
+			STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_wait");
 		}
 	}
 
