@@ -1,7 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2013-2015,2017,2018                      CNRS
- * Copyright (C) 2014,2018                                Université de Bordeaux
+ * Copyright (C) 2013-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -31,11 +30,11 @@ int *starpu_value_get(starpu_data_handle_t handle)
 	return value_interface->value;
 }
 
-static void value_register_data_handle(starpu_data_handle_t handle, unsigned home_node, void *data_interface)
+static void value_register_data_handle(starpu_data_handle_t handle, int home_node, void *data_interface)
 {
 	struct starpu_value_interface *value_interface = (struct starpu_value_interface *) data_interface;
 
-	unsigned node;
+	int node;
 	for (node = 0; node < STARPU_MAXNODES; node++)
 	{
 		struct starpu_value_interface *local_interface = (struct starpu_value_interface *)
@@ -68,6 +67,7 @@ static void value_free_data_on_node(void *data_interface, unsigned node)
 	struct starpu_value_interface *value_interface = (struct starpu_value_interface *) data_interface;
 
 	starpu_free_on_node(node, (uintptr_t) value_interface->value, sizeof(int));
+	value_interface->value = NULL;
 }
 
 static size_t value_get_size(starpu_data_handle_t handle)
@@ -99,14 +99,14 @@ static int value_pack_data(starpu_data_handle_t handle, unsigned node, void **pt
 	*count = sizeof(int);
 	if (ptr != NULL)
 	{
-		*ptr = malloc(*count);
+		*ptr = (void*) starpu_malloc_on_node_flags(node, *count, 0);
 		memcpy(*ptr, value_interface->value, sizeof(int));
 	}
 
 	return 0;
 }
 
-static int value_unpack_data(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
+static int value_peek_data(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
 {
 	(void)count;
 	STARPU_ASSERT(starpu_data_test_if_allocated_on_node(handle, node));
@@ -121,6 +121,14 @@ static int value_unpack_data(starpu_data_handle_t handle, unsigned node, void *p
 	return 0;
 }
 
+static int value_unpack_data(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
+{
+	value_peek_data(handle, node, ptr, count);
+	starpu_free_on_node_flags(node, (uintptr_t)ptr, count, 0);
+
+	return 0;
+}
+
 static int copy_any_to_any(void *src_interface, unsigned src_node,
 			   void *dst_interface, unsigned dst_node,
 			   void *async_data)
@@ -151,6 +159,7 @@ static struct starpu_data_interface_ops interface_value_ops =
 	.interface_size = sizeof(struct starpu_value_interface),
 	.to_pointer = value_to_pointer,
 	.pack_data = value_pack_data,
+	.peek_data = value_peek_data,
 	.unpack_data = value_unpack_data
 };
 
@@ -161,11 +170,6 @@ void starpu_value_data_register(starpu_data_handle_t *handleptr, unsigned home_n
 		.value = value
 	};
 
-	if (interface_value_ops.interfaceid == STARPU_UNKNOWN_INTERFACE_ID)
-	{
-		interface_value_ops.interfaceid = starpu_data_interface_get_next_id();
-	}
-
 	starpu_data_register(handleptr, home_node, &value_int, &interface_value_ops);
 }
 
