@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2014                                Inria
- * Copyright (C) 2012-2017                                CNRS
- * Copyright (C) 2012                                     Université de Bordeaux
+ * Copyright (C) 2011-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -103,7 +101,7 @@ static void _size_ctxs(unsigned *sched_ctxs, int nsched_ctxs , int *workers, int
 
 	/* smallest possible tmax, difficult to obtain as we
 	   compute the nr of flops and not the tasks */
-        /*lp computes it in s but it's converted to ms just before return */
+	/*lp computes it in s but it's converted to ms just before return */
 	double possible_tmax = sc_hypervisor_lp_get_tmax(nw, workers);
 	double smallest_tmax = possible_tmax / 3;
 	double tmax = possible_tmax * ns;
@@ -122,6 +120,7 @@ static void _size_ctxs(unsigned *sched_ctxs, int nsched_ctxs , int *workers, int
 	{
 		struct types_of_workers *tw = sc_hypervisor_get_types_of_workers(workers, nw);
 		sc_hypervisor_lp_place_resources_in_ctx(ns, nw, w_in_s, sched_ctxs, workers, 1, tw);
+		free(tw);
 	}
 
 	for(i = 0; i < nw; i++)
@@ -209,7 +208,7 @@ static void _try_resizing(unsigned *sched_ctxs, int nsched_ctxs , int *workers,
 
 	/* smallest possible tmax, difficult to obtain as we
 	   compute the nr of flops and not the tasks */
-        /*lp computes it in s but it's converted to ms just before return */
+	/*lp computes it in s but it's converted to ms just before return */
 	double possible_tmax = sc_hypervisor_lp_get_tmax(nw, NULL);
 	double smallest_tmax = possible_tmax/2.0;
 	double tmax = possible_tmax + smallest_tmax;
@@ -228,7 +227,7 @@ static void _try_resizing(unsigned *sched_ctxs, int nsched_ctxs , int *workers,
 		gettimeofday(&end_time, NULL);
 
 		long diff_s = end_time.tv_sec  - start_time.tv_sec;
-		long diff_us = end_time.tv_usec  - start_time.tv_usec;
+		long diff_us = end_time.tv_usec	 - start_time.tv_usec;
 
 		__attribute__((unused))	float timing = (float)(diff_s*1000000 + diff_us)/1000.0;
 	}
@@ -239,6 +238,7 @@ static void _try_resizing(unsigned *sched_ctxs, int nsched_ctxs , int *workers,
 	{
 		struct types_of_workers *tw = sc_hypervisor_get_types_of_workers(workers, nw);
 		sc_hypervisor_lp_place_resources_in_ctx(ns, nw, w_in_s, sched_ctxs, workers, 0, tw);
+		free(tw);
 	}
 
 	struct sc_hypervisor_policy_task_pool *next = NULL;
@@ -257,10 +257,9 @@ static void _try_resizing(unsigned *sched_ctxs, int nsched_ctxs , int *workers,
 
 static void teft_lp_handle_poped_task(unsigned sched_ctx, int worker, struct starpu_task *task, uint32_t footprint)
 {
+	(void)sched_ctx;
 	if(worker > -2)
 	{
-		struct sc_hypervisor_wrapper* sc_w = sc_hypervisor_get_wrapper(sched_ctx);
-		
 		int ret = starpu_pthread_mutex_trylock(&act_hypervisor_mutex);
 		if(ret != EBUSY)
 		{
@@ -286,11 +285,11 @@ static void teft_lp_handle_poped_task(unsigned sched_ctx, int worker, struct sta
 
 static void teft_lp_handle_idle_cycle(unsigned sched_ctx, int worker)
 {
+	(void)sched_ctx;
+	(void)worker;
 	unsigned criteria = sc_hypervisor_get_resize_criteria();
 	if(criteria != SC_NOTHING)// && criteria == SC_IDLE)
 	{
-		struct sc_hypervisor_wrapper* sc_w = sc_hypervisor_get_wrapper(sched_ctx);
-		
 		int ret = starpu_pthread_mutex_trylock(&act_hypervisor_mutex);
 		if(ret != EBUSY)
 		{
@@ -330,7 +329,8 @@ static void teft_lp_resize_ctxs(unsigned *sched_ctxs, int nsched_ctxs , int *wor
 	}
 }
 
-struct sc_hypervisor_policy teft_lp_policy = {
+struct sc_hypervisor_policy teft_lp_policy =
+{
 	.size_ctxs = teft_lp_size_ctxs,
 	.resize_ctxs = teft_lp_resize_ctxs,
 	.handle_poped_task = teft_lp_handle_poped_task,
@@ -340,6 +340,7 @@ struct sc_hypervisor_policy teft_lp_policy = {
 	.handle_post_exec_hook = NULL,
 	.handle_submitted_job = teft_lp_handle_submitted_job,
 	.end_ctx = NULL,
+	.init_worker = NULL,
 	.custom = 0,
 	.name = "teft_lp"
 };
