@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2018       Federal University of Rio Grande do Sul (UFRGS)
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -23,6 +23,7 @@
 #include <common/fxt.h>
 #include <starpu.h>
 #include <drivers/opencl/driver_opencl.h>
+#include <drivers/driver_common/driver_common.h>
 #include <datawizard/memory_manager.h>
 #include <datawizard/memory_nodes.h>
 #include <datawizard/malloc.h>
@@ -55,12 +56,22 @@
 
 static size_t _malloc_align = sizeof(void*);
 static int disable_pinning;
-static int malloc_on_node_default_flags[STARPU_MAXNODES];
+static int enable_suballocator;
 
 /* This file is used for implementing "folded" allocation */
 #ifdef STARPU_SIMGRID
 static int bogusfile = -1;
 static unsigned long _starpu_malloc_simulation_fold;
+/* Table to control unique simulation mallocs */
+#include <common/uthash.h>
+struct unique_shared_alloc
+{
+        size_t id;
+        int count;
+        void* addr;
+        UT_hash_handle hh;
+};
+static struct unique_shared_alloc* unique_shared_alloc_table = NULL;
 #endif
 
 static starpu_malloc_hook malloc_hook;
@@ -79,6 +90,8 @@ void starpu_malloc_set_align(size_t align)
 		_malloc_align = align;
 }
 
+/* Driver porters: adding your driver here is optional, only needed for pinning host memory.  */
+
 #if (defined(STARPU_USE_CUDA) && !defined(STARPU_HAVE_CUDA_MEMCPY_PEER))// || defined(STARPU_USE_OPENCL)
 struct malloc_pinned_codelet_struct
 {
@@ -102,9 +115,18 @@ struct malloc_pinned_codelet_struct
 static void malloc_pinned_cuda_codelet(void *buffers[] STARPU_ATTRIBUTE_UNUSED, void *arg)
 {
 	struct malloc_pinned_codelet_struct *s = arg;
+	cudaError_t cures = cudaErrorMemoryAllocation;
+#if 0 //defined(STARPU_USE_CUDA_MAP) && defined(STARPU_HAVE_CUDA_MNGMEM)
+	/* FIXME: check if devices actually support cudaMallocManaged or fallback to cudaHostAlloc() */
+	cures = cudaMallocManaged((void **)(s->ptr), s->dim, cudaMemAttachGlobal);
+#endif
+#if defined(STARPU_USE_CUDA_MAP) && defined(STARPU_HAVE_CUDA_CANMAPHOST)
+	if (cures != cudaSuccess)
+		cures = cudaHostAlloc((void **)(s->ptr), s->dim, cudaHostAllocPortable|cudaHostAllocMapped);
+#endif
+	if (cures != cudaSuccess)
+		cures = cudaHostAlloc((void **)(s->ptr), s->dim, cudaHostAllocPortable);
 
-	cudaError_t cures;
-	cures = cudaHostAlloc((void **)(s->ptr), s->dim, cudaHostAllocPortable);
 	if (STARPU_UNLIKELY(cures))
 		STARPU_CUDA_REPORT_ERROR(cures);
 }
@@ -143,6 +165,10 @@ static int _starpu_malloc_should_pin(int flags)
 		{
 			return 1;
 		}
+		if (_starpu_can_submit_hip_task())
+		{
+			return 1;
+		}
 //		if (_starpu_can_submit_opencl_task())
 //			return 1;
 	}
@@ -151,9 +177,10 @@ static int _starpu_malloc_should_pin(int flags)
 
 int _starpu_malloc_willpin_on_node(unsigned dst_node)
 {
-	int flags = malloc_on_node_default_flags[dst_node];
+	int flags = _starpu_get_node_struct(dst_node)->malloc_on_node_default_flags;
 	return (_starpu_malloc_should_pin(flags) && STARPU_RUNNING_ON_VALGRIND == 0
 			&& (_starpu_can_submit_cuda_task()
+			    || _starpu_can_submit_hip_task()
 			    /* || _starpu_can_submit_opencl_task() */
 			));
 }
@@ -162,7 +189,13 @@ int _starpu_malloc_flags_on_node(unsigned dst_node, void **A, size_t dim, int fl
 {
 	int ret=0;
 
-	STARPU_ASSERT(A);
+	STARPU_ASSERT_MSG(A, "starpu_malloc needs to be passed the address of the pointer to be filled");
+	if (!starpu_is_initialized())
+		_STARPU_DISP("Warning: starpu_malloc needs to be called after starpu is initialized, to be able to pin memory for CUDA\n");
+
+	if (dim == 0)
+		/* Make sure we succeed */
+		dim = 1;
 
 	if (flags & STARPU_MALLOC_COUNT)
 	{
@@ -173,7 +206,7 @@ int _starpu_malloc_flags_on_node(unsigned dst_node, void **A, size_t dim, int fl
 				size_t reclaim = 2 * dim;
 				_STARPU_DEBUG("There is not enough memory left, we are going to reclaim %ld\n", (long)reclaim);
 				_STARPU_TRACE_START_MEMRECLAIM(dst_node,0);
-				freed = _starpu_memory_reclaim_generic(dst_node, 0, reclaim);
+				freed = _starpu_memory_reclaim_generic(dst_node, 0, reclaim, STARPU_FETCH);
 				_STARPU_TRACE_END_MEMRECLAIM(dst_node,0);
 				if (freed < dim && !(flags & STARPU_MEMORY_WAIT))
 				{
@@ -209,8 +242,21 @@ int _starpu_malloc_flags_on_node(unsigned dst_node, void **A, size_t dim, int fl
 #else /* STARPU_SIMGRID */
 #ifdef STARPU_USE_CUDA
 #ifdef STARPU_HAVE_CUDA_MEMCPY_PEER
-			cudaError_t cures;
-			cures = cudaHostAlloc(A, dim, cudaHostAllocPortable);
+			cudaError_t cures = cudaErrorMemoryAllocation;
+
+#if 0 //defined(STARPU_USE_CUDA_MAP) && defined(STARPU_HAVE_CUDA_MNGMEM)
+			/* FIXME: check if devices actually support cudaMallocManaged or fallback to cudaHostAlloc() */
+			cures = cudaMallocManaged(A, dim, cudaMemAttachGlobal);
+#endif
+
+#if defined(STARPU_USE_CUDA_MAP) && defined(STARPU_HAVE_CUDA_CANMAPHOST)
+			if (cures != cudaSuccess)
+				cures = cudaHostAlloc(A, dim, cudaHostAllocPortable|cudaHostAllocMapped);
+#endif
+
+			if (cures != cudaSuccess)
+				cures = cudaHostAlloc(A, dim, cudaHostAllocPortable);
+
 			if (STARPU_UNLIKELY(cures))
 			{
 				STARPU_CUDA_REPORT_ERROR(cures);
@@ -247,6 +293,34 @@ int _starpu_malloc_flags_on_node(unsigned dst_node, void **A, size_t dim, int fl
 			goto end;
 #endif /* STARPU_HAVE_CUDA_MEMCPY_PEER */
 #endif /* STARPU_USE_CUDA */
+		}
+		if (_starpu_can_submit_hip_task())
+		{
+#ifdef STARPU_USE_HIP
+			hipError_t hipres = hipErrorMemoryAllocation;
+
+#if 0 //defined(STARPU_USE_HIP_MAP) && defined(STARPU_HAVE_HIP_MNGMEM)
+			/* FIXME: check if devices actually support hipMallocManaged or fallback to hipHostAlloc() */
+			hipres = hipMallocManaged(A, dim, hipMemAttachGlobal);
+#endif
+
+#if defined(STARPU_USE_HIP_MAP) && defined(STARPU_HAVE_HIP_CANMAPHOST)
+			if (hipres != hipSuccess)
+				hipres = hipHostMalloc(A, dim, hipHostMallocPortable|hipHostMallocMapped);
+#endif
+
+			if (hipres != hipSuccess)
+			{
+				hipres = hipHostMalloc(A, dim, hipHostMallocPortable);
+			}
+
+			if (STARPU_UNLIKELY(hipres != hipSuccess))
+			{
+				STARPU_HIP_REPORT_ERROR(hipres);
+				ret = -ENOMEM;
+			}
+			goto end;
+#endif /* STARPU_USE_HIP */
 //		}
 //		else if (_starpu_can_submit_opencl_task())
 //		{
@@ -284,11 +358,34 @@ int _starpu_malloc_flags_on_node(unsigned dst_node, void **A, size_t dim, int fl
 	if (flags & STARPU_MALLOC_SIMULATION_FOLDED)
 	{
 #if SIMGRID_VERSION >= 31500 && SIMGRID_VERSION != 31559
-	    if (_starpu_simgrid_running_smpi())
-		    *A = SMPI_SHARED_MALLOC(dim);
-	    else
+		if(_starpu_simgrid_running_smpi())
+		{
+			if(flags & STARPU_MALLOC_SIMULATION_UNIQUE)
+			{
+				struct unique_shared_alloc *block;
+				HASH_FIND(hh, unique_shared_alloc_table, &dim, sizeof(dim), block);
+				if(block==NULL)
+				{
+					block = (struct unique_shared_alloc*)malloc(sizeof(struct unique_shared_alloc));
+					block->addr = SMPI_SHARED_MALLOC(dim);
+					block->count = 1;
+					block->id = dim;
+					HASH_ADD(hh, unique_shared_alloc_table, id, sizeof(dim), block);
+				}
+				else
+				{
+					block->count++;
+				}
+				*A = block->addr;
+			}
+			else
+			{
+				*A = SMPI_SHARED_MALLOC(dim);
+			}
+		}
+		else
 #endif
-	    {
+		{
 		/* Use "folded" allocation: the same file is mapped several
 		 * times contiguously, to get a memory area one can read/write,
 		 * without consuming memory */
@@ -348,6 +445,36 @@ int _starpu_malloc_flags_on_node(unsigned dst_node, void **A, size_t dim, int fl
 	    }
 	}
 #endif
+
+#ifdef HAVE_MMAP
+#ifdef STARPU_USE_MP
+	if(_starpu_can_submit_ms_task())
+	{
+		*A = _starpu_map_allocate(dim, dst_node);
+
+		if (!*A)
+			ret = -ENOMEM;
+		else
+		{
+#ifdef STARPU_HAVE_HWLOC
+			struct _starpu_machine_config *config = _starpu_get_machine_config();
+			hwloc_topology_t hwtopology = config->topology.hwtopology;
+			hwloc_obj_t numa_node_obj = hwloc_get_obj_by_type(hwtopology, HWLOC_OBJ_NUMANODE, starpu_memory_nodes_numa_id_to_hwloclogid(dst_node));
+			if (numa_node_obj)
+			{
+				hwloc_bitmap_t nodeset = numa_node_obj->nodeset;
+#if HWLOC_API_VERSION >= 0x00020000
+				hwloc_set_area_membind(hwtopology, *A, dim, nodeset, HWLOC_MEMBIND_BIND, HWLOC_MEMBIND_BYNODESET | HWLOC_MEMBIND_NOCPUBIND);
+#else
+				hwloc_set_area_membind_nodeset(hwtopology, *A, dim, nodeset, HWLOC_MEMBIND_BIND, HWLOC_MEMBIND_NOCPUBIND);
+#endif
+			}
+#endif
+		}
+	}
+	else
+#endif
+#endif
 #ifdef STARPU_HAVE_HWLOC
 	if (starpu_memory_nodes_get_numa_count() > 1)
 	{
@@ -413,7 +540,12 @@ int starpu_malloc(void **A, size_t dim)
 static void free_pinned_cuda_codelet(void *buffers[] STARPU_ATTRIBUTE_UNUSED, void *arg)
 {
 	cudaError_t cures;
+#if 0 //defined(STARPU_USE_CUDA_MAP) && defined(STARPU_HAVE_CUDA_MNGMEM)
+	/* FIXME: check if devices actually support cudaMallocManaged or fallback to cudaHostAlloc() */
+	cures = cudaFree(arg);
+#else
 	cures = cudaFreeHost(arg);
+#endif
 	if (STARPU_UNLIKELY(cures))
 		STARPU_CUDA_REPORT_ERROR(cures);
 }
@@ -453,6 +585,9 @@ int starpu_free_flags(void *A, size_t dim, int flags)
 
 int _starpu_free_flags_on_node(unsigned dst_node, void *A, size_t dim, int flags)
 {
+	if (dim == 0)
+		dim = 1;
+
 	if (free_hook)
 	{
 		free_hook(dst_node, A, dim, flags);
@@ -474,9 +609,15 @@ int _starpu_free_flags_on_node(unsigned dst_node, void *A, size_t dim, int flags
 				/* This is especially useful when starpu_free is called even
 				 * though starpu_shutdown has already
 				 * been called, so we will not be able to submit a task. */
-				cudaError_t err = cudaFreeHost(A);
-				if (STARPU_UNLIKELY(err))
-					STARPU_CUDA_REPORT_ERROR(err);
+				cudaError_t cures;
+#if 0 //defined(STARPU_USE_CUDA_MAP) && defined(STARPU_HAVE_CUDA_MNGMEM)
+				/* FIXME: check if devices actually support cudaMallocManaged or fallback to cudaHostAlloc() */
+				cures = cudaFree(A);
+#else
+				cures = cudaFreeHost(A);
+#endif
+				if (STARPU_UNLIKELY(cures))
+					STARPU_CUDA_REPORT_ERROR(cures);
 				goto out;
 #ifndef STARPU_HAVE_CUDA_MEMCPY_PEER
 			}
@@ -503,6 +644,20 @@ int _starpu_free_flags_on_node(unsigned dst_node, void *A, size_t dim, int flags
 			}
 #endif /* STARPU_HAVE_CUDA_MEMCPY_PEER */
 #endif /* STARPU_USE_CUDA */
+		}
+		if (_starpu_can_submit_hip_task())
+		{
+#ifdef STARPU_USE_HIP
+			/* TODO: submit task */
+			/* This is especially useful when starpu_free is called even
+			 * though starpu_shutdown has already
+			 * been called, so we will not be able to submit a task. */
+			hipError_t hipres;
+			hipres = hipHostFree(A);
+			if (STARPU_UNLIKELY(hipres))
+				STARPU_HIP_REPORT_ERROR(hipres);
+			goto out;
+#endif /* STARPU_USE_HIP */
 #endif /* STARPU_SIMGRID */
 		}
 //	else if (_starpu_can_submit_opencl_task())
@@ -534,13 +689,39 @@ int _starpu_free_flags_on_node(unsigned dst_node, void *A, size_t dim, int flags
 	if (flags & STARPU_MALLOC_SIMULATION_FOLDED)
 	{
 #if SIMGRID_VERSION >= 31500 && SIMGRID_VERSION != 31559
-	    if (_starpu_simgrid_running_smpi())
-		SMPI_SHARED_FREE(A);
-	    else
+		if(_starpu_simgrid_running_smpi())
+		{
+			if(flags & STARPU_MALLOC_SIMULATION_UNIQUE)
+			{
+				struct unique_shared_alloc *block;
+				HASH_FIND(hh, unique_shared_alloc_table, &dim, sizeof(dim), block);
+				STARPU_ASSERT(block != NULL);
+				block->count--;
+				if(block->count == 0)
+				{
+					SMPI_SHARED_FREE(block->addr);
+					HASH_DEL(unique_shared_alloc_table, block);
+					free(block);
+				}
+			}
+			else
+			{
+				SMPI_SHARED_FREE(A);
+			}
+		}
+		else
 #endif
-		munmap(A, dim);
+			munmap(A, dim);
 	}
 #endif
+#ifdef HAVE_MMAP
+#ifdef STARPU_USE_MP
+	else if(_starpu_can_submit_ms_task())
+	{
+		_starpu_map_deallocate(A, dim);
+	}
+#endif
+#endif
 #ifdef STARPU_HAVE_HWLOC
 	else if (starpu_memory_nodes_get_numa_count() > 1)
 	{
@@ -566,10 +747,18 @@ int starpu_free(void *A)
 	return starpu_free_flags(A, 0, STARPU_MALLOC_PINNED);
 }
 
+int starpu_free_noflag(void *A, size_t dim)
+{
+	return starpu_free_flags(A, dim, STARPU_MALLOC_PINNED);
+}
+
 static uintptr_t _starpu_malloc_on_node(unsigned dst_node, size_t size, int flags)
 {
 	uintptr_t addr = 0;
 
+	if (size == 0)
+		size = 1;
+
 	/* Handle count first */
 	if (flags & STARPU_MALLOC_COUNT)
 	{
@@ -579,7 +768,7 @@ static uintptr_t _starpu_malloc_on_node(unsigned dst_node, size_t size, int flag
 		flags &= ~STARPU_MALLOC_COUNT;
 	}
 
-	struct _starpu_node_ops *node_ops = _starpu_memory_node_get_node_ops(dst_node);
+	const struct _starpu_node_ops *node_ops = _starpu_memory_node_get_node_ops(dst_node);
 	if (node_ops && node_ops->malloc_on_node)
 		addr = node_ops->malloc_on_node(dst_node, size, flags & ~STARPU_MALLOC_COUNT);
 	else
@@ -600,7 +789,10 @@ void _starpu_free_on_node_flags(unsigned dst_node, uintptr_t addr, size_t size,
 	int count = flags & STARPU_MALLOC_COUNT;
 	flags &= ~STARPU_MALLOC_COUNT;
 
-	struct _starpu_node_ops *node_ops = _starpu_memory_node_get_node_ops(dst_node);
+	if (size == 0)
+		size = 1;
+
+	const struct _starpu_node_ops *node_ops = _starpu_memory_node_get_node_ops(dst_node);
 	if (node_ops && node_ops->free_on_node)
 		node_ops->free_on_node(dst_node, addr, size, flags);
 	else
@@ -618,6 +810,10 @@ starpu_memory_pin(void *addr STARPU_ATTRIBUTE_UNUSED, size_t size STARPU_ATTRIBU
 #if defined(STARPU_USE_CUDA) && defined(STARPU_HAVE_CUDA_MEMCPY_PEER)
 		if (cudaHostRegister(addr, size, cudaHostRegisterPortable) != cudaSuccess)
 			return -1;
+#endif
+#if defined(STARPU_USE_HIP)
+		if (hipHostRegister(addr, size, hipHostRegisterPortable) != hipSuccess)
+			return -1;
 #endif
 	}
 	return 0;
@@ -631,100 +827,49 @@ starpu_memory_unpin(void *addr STARPU_ATTRIBUTE_UNUSED, size_t size STARPU_ATTRI
 #if defined(STARPU_USE_CUDA) && defined(STARPU_HAVE_CUDA_MEMCPY_PEER)
 		if (cudaHostUnregister(addr) != cudaSuccess)
 			return -1;
+#endif
+#if defined(STARPU_USE_HIP)
+		if (hipHostUnregister(addr) != hipSuccess)
+			return -1;
 #endif
 	}
 	return 0;
 }
 
-/*
- * On CUDA which has very expensive malloc, for small sizes, allocate big
- * chunks divided in blocks, and we actually allocate segments of consecutive
- * blocks.
- *
- * We try to keep the list of chunks with increasing occupancy, so we can
- * quickly find free segments to allocate.
- */
-
-/* Size of each chunk, 32MiB granularity brings 128 chunks to be allocated in
- * order to fill a 4GiB GPU. */
-#define CHUNK_SIZE (32*1024*1024)
-
-/* Maximum segment size we will allocate in chunks */
-#define CHUNK_ALLOC_MAX (CHUNK_SIZE / 8)
-
-/* Granularity of allocation, i.e. block size, StarPU will never allocate less
- * than this.
- * 16KiB (i.e. 64x64 float) granularity eats 2MiB RAM for managing a 4GiB GPU.
- */
-#define CHUNK_ALLOC_MIN (16*1024)
-
-/* Don't really deallocate chunks unless we have more than this many chunks
- * which are completely free. */
-#define CHUNKS_NFREE 4
-
-/* Number of blocks */
-#define CHUNK_NBLOCKS (CHUNK_SIZE/CHUNK_ALLOC_MIN)
-
-/* Linked list for available segments */
-struct block
-{
-	int length;	/* Number of consecutive free blocks */
-	int next;	/* next free segment */
-};
-
-/* One chunk */
-LIST_TYPE(_starpu_chunk,
-	uintptr_t base;
-
-	/* Available number of blocks, for debugging */
-	int available;
-
-	/* Overestimation of the maximum size of available segments in this chunk */
-	int available_max;
-
-	/* Bitmap describing availability of the block */
-	/* Block 0 is always empty, and is just the head of the free segments list */
-	struct block bitmap[CHUNK_NBLOCKS+1];
-)
-
-/* One list of chunks per node */
-static struct _starpu_chunk_list chunks[STARPU_MAXNODES];
-/* Number of completely free chunks */
-static int nfreechunks[STARPU_MAXNODES];
-/* This protects chunks and nfreechunks */
-static starpu_pthread_mutex_t chunk_mutex[STARPU_MAXNODES];
-
 void
 _starpu_malloc_init(unsigned dst_node)
 {
-	_starpu_chunk_list_init(&chunks[dst_node]);
-	nfreechunks[dst_node] = 0;
-	STARPU_PTHREAD_MUTEX_INIT(&chunk_mutex[dst_node], NULL);
-	disable_pinning = starpu_get_env_number("STARPU_DISABLE_PINNING");
-	malloc_on_node_default_flags[dst_node] = STARPU_MALLOC_PINNED | STARPU_MALLOC_COUNT;
+	struct _starpu_node *node_struct = _starpu_get_node_struct(dst_node);
+	_starpu_chunk_list_init(&node_struct->chunks);
+	node_struct->nfreechunks = 0;
+	STARPU_PTHREAD_MUTEX_INIT(&node_struct->chunk_mutex, NULL);
+	disable_pinning = starpu_getenv_number("STARPU_DISABLE_PINNING");
+	enable_suballocator = starpu_getenv_number_default("STARPU_SUBALLOCATOR", 1);
+	node_struct->malloc_on_node_default_flags = STARPU_MALLOC_PINNED | STARPU_MALLOC_COUNT;
 #ifdef STARPU_SIMGRID
 	/* Reasonably "costless" */
-	_starpu_malloc_simulation_fold = starpu_get_env_number_default("STARPU_MALLOC_SIMULATION_FOLD", 1) << 20;
+	_starpu_malloc_simulation_fold = starpu_getenv_number_default("STARPU_MALLOC_SIMULATION_FOLD", 1) << 20;
 #endif
 }
 
 void
 _starpu_malloc_shutdown(unsigned dst_node)
 {
+	struct _starpu_node *node_struct = _starpu_get_node_struct(dst_node);
 	struct _starpu_chunk *chunk, *next_chunk;
 
-	STARPU_PTHREAD_MUTEX_LOCK(&chunk_mutex[dst_node]);
-	for (chunk = _starpu_chunk_list_begin(&chunks[dst_node]);
-	     chunk != _starpu_chunk_list_end(&chunks[dst_node]);
+	STARPU_PTHREAD_MUTEX_LOCK(&node_struct->chunk_mutex);
+	for (chunk = _starpu_chunk_list_begin(&node_struct->chunks);
+	     chunk != _starpu_chunk_list_end(&node_struct->chunks);
 	     chunk = next_chunk)
 	{
 		next_chunk = _starpu_chunk_list_next(chunk);
-		_starpu_free_on_node_flags(dst_node, chunk->base, CHUNK_SIZE, malloc_on_node_default_flags[dst_node]);
-		_starpu_chunk_list_erase(&chunks[dst_node], chunk);
+		_starpu_free_on_node_flags(dst_node, chunk->base, CHUNK_SIZE, node_struct->malloc_on_node_default_flags);
+		_starpu_chunk_list_erase(&node_struct->chunks, chunk);
 		free(chunk);
 	}
-	STARPU_PTHREAD_MUTEX_UNLOCK(&chunk_mutex[dst_node]);
-	STARPU_PTHREAD_MUTEX_DESTROY(&chunk_mutex[dst_node]);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&node_struct->chunk_mutex);
+	STARPU_PTHREAD_MUTEX_DESTROY(&node_struct->chunk_mutex);
 }
 
 /* Create a new chunk */
@@ -756,11 +901,13 @@ static struct _starpu_chunk *_starpu_new_chunk(unsigned dst_node, int flags)
 /* Return whether we should use our suballocator */
 static int _starpu_malloc_should_suballoc(unsigned dst_node, size_t size, int flags)
 {
-	return size <= CHUNK_ALLOC_MAX &&
+	return (enable_suballocator &&
+		(size <= CHUNK_ALLOC_MAX &&
 		(starpu_node_get_kind(dst_node) == STARPU_CUDA_RAM
 		 || (starpu_node_get_kind(dst_node) == STARPU_CPU_RAM
 		     && _starpu_malloc_should_pin(flags))
-		 );
+		 )))
+	       || starpu_node_get_kind(dst_node) == STARPU_MAX_FPGA_RAM;
 }
 
 uintptr_t
@@ -770,19 +917,23 @@ starpu_malloc_on_node_flags(unsigned dst_node, size_t size, int flags)
 	if (!_starpu_malloc_should_suballoc(dst_node, size, flags))
 		return _starpu_malloc_on_node(dst_node, size, flags);
 
+	struct _starpu_node *node_struct = _starpu_get_node_struct(dst_node);
+
 	/* Round up allocation to block size */
 	int nblocks = (size + CHUNK_ALLOC_MIN - 1) / CHUNK_ALLOC_MIN;
+	if (!nblocks)
+		nblocks = 1;
 
 	struct _starpu_chunk *chunk;
 	int prevblock, block;
 	int available_max;
 	struct block *bitmap;
 
-	STARPU_PTHREAD_MUTEX_LOCK(&chunk_mutex[dst_node]);
+	STARPU_PTHREAD_MUTEX_LOCK(&node_struct->chunk_mutex);
 
 	/* Try to find a big enough segment among the chunks */
-	for (chunk = _starpu_chunk_list_begin(&chunks[dst_node]);
-	     chunk != _starpu_chunk_list_end(&chunks[dst_node]);
+	for (chunk = _starpu_chunk_list_begin(&node_struct->chunks);
+	     chunk != _starpu_chunk_list_end(&node_struct->chunks);
 	     chunk = _starpu_chunk_list_next(chunk))
 	{
 		if (chunk->available_max < nblocks)
@@ -804,12 +955,12 @@ starpu_malloc_on_node_flags(unsigned dst_node, size_t size, int flags)
 					/* This one this has quite some room,
 					 * put it front, to make finding it
 					 * easier next time. */
-					_starpu_chunk_list_erase(&chunks[dst_node], chunk);
-					_starpu_chunk_list_push_front(&chunks[dst_node], chunk);
+					_starpu_chunk_list_erase(&node_struct->chunks, chunk);
+					_starpu_chunk_list_push_front(&node_struct->chunks, chunk);
 				}
 				if (chunk->available == CHUNK_NBLOCKS)
 					/* This one was empty, it's not empty any more */
-					nfreechunks[dst_node]--;
+					node_struct->nfreechunks--;
 				goto found;
 			}
 			if (length > available_max)
@@ -826,13 +977,13 @@ starpu_malloc_on_node_flags(unsigned dst_node, size_t size, int flags)
 	if (!chunk)
 	{
 		/* Really no memory any more, fail */
-		STARPU_PTHREAD_MUTEX_UNLOCK(&chunk_mutex[dst_node]);
+		STARPU_PTHREAD_MUTEX_UNLOCK(&node_struct->chunk_mutex);
 		errno = ENOMEM;
 		return 0;
 	}
 
 	/* And make it easy to find. */
-	_starpu_chunk_list_push_front(&chunks[dst_node], chunk);
+	_starpu_chunk_list_push_front(&node_struct->chunks, chunk);
 	bitmap = chunk->bitmap;
 	prevblock = 0;
 	block = 1;
@@ -856,7 +1007,7 @@ found:
 		bitmap[block + nblocks].next = bitmap[block].next;
 	}
 
-	STARPU_PTHREAD_MUTEX_UNLOCK(&chunk_mutex[dst_node]);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&node_struct->chunk_mutex);
 
 	return chunk->base + (block-1) * CHUNK_ALLOC_MIN;
 }
@@ -871,18 +1022,21 @@ starpu_free_on_node_flags(unsigned dst_node, uintptr_t addr, size_t size, int fl
 		return;
 	}
 
+	struct _starpu_node *node_struct = _starpu_get_node_struct(dst_node);
 	struct _starpu_chunk *chunk;
 
 	/* Round up allocation to block size */
 	int nblocks = (size + CHUNK_ALLOC_MIN - 1) / CHUNK_ALLOC_MIN;
+	if (!nblocks)
+		nblocks = 1;
 
-	STARPU_PTHREAD_MUTEX_LOCK(&chunk_mutex[dst_node]);
-	for (chunk = _starpu_chunk_list_begin(&chunks[dst_node]);
-	     chunk != _starpu_chunk_list_end(&chunks[dst_node]);
+	STARPU_PTHREAD_MUTEX_LOCK(&node_struct->chunk_mutex);
+	for (chunk = _starpu_chunk_list_begin(&node_struct->chunks);
+	     chunk != _starpu_chunk_list_end(&node_struct->chunks);
 	     chunk = _starpu_chunk_list_next(chunk))
 		if (addr >= chunk->base && addr < chunk->base + CHUNK_SIZE)
 			break;
-	STARPU_ASSERT(chunk != _starpu_chunk_list_end(&chunks[dst_node]));
+	STARPU_ASSERT(chunk != _starpu_chunk_list_end(&node_struct->chunks));
 
 	struct block *bitmap = chunk->bitmap;
 	int block = ((addr - chunk->base) / CHUNK_ALLOC_MIN) + 1, prevblock, nextblock;
@@ -934,40 +1088,41 @@ starpu_free_on_node_flags(unsigned dst_node, uintptr_t addr, size_t size, int fl
 	{
 		/* This chunk is now empty, but avoid chunk free/alloc
 		 * ping-pong by keeping some of these.  */
-		if (nfreechunks[dst_node] >= CHUNKS_NFREE)
+		if (node_struct->nfreechunks >= CHUNKS_NFREE &&
+		     starpu_node_get_kind(dst_node) != STARPU_MAX_FPGA_RAM)
 		{
 			/* We already have free chunks, release this one */
 			_starpu_free_on_node_flags(dst_node, chunk->base, CHUNK_SIZE, flags);
-			_starpu_chunk_list_erase(&chunks[dst_node], chunk);
+			_starpu_chunk_list_erase(&node_struct->chunks, chunk);
 			free(chunk);
 		}
 		else
-			nfreechunks[dst_node]++;
+			node_struct->nfreechunks++;
 	}
 	else
 	{
 		/* Freed some room, put this first in chunks list */
-		_starpu_chunk_list_erase(&chunks[dst_node], chunk);
-		_starpu_chunk_list_push_front(&chunks[dst_node], chunk);
+		_starpu_chunk_list_erase(&node_struct->chunks, chunk);
+		_starpu_chunk_list_push_front(&node_struct->chunks, chunk);
 	}
 
-	STARPU_PTHREAD_MUTEX_UNLOCK(&chunk_mutex[dst_node]);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&node_struct->chunk_mutex);
 }
 
 void starpu_malloc_on_node_set_default_flags(unsigned node, int flags)
 {
 	STARPU_ASSERT_MSG(node < STARPU_MAXNODES, "bogus node value %u given to starpu_malloc_on_node_set_default_flags\n", node);
-	malloc_on_node_default_flags[node] = flags;
+	_starpu_get_node_struct(node)->malloc_on_node_default_flags = flags;
 }
 
 uintptr_t
 starpu_malloc_on_node(unsigned dst_node, size_t size)
 {
-	return starpu_malloc_on_node_flags(dst_node, size, malloc_on_node_default_flags[dst_node]);
+	return starpu_malloc_on_node_flags(dst_node, size, _starpu_get_node_struct(dst_node)->malloc_on_node_default_flags);
 }
 
 void
 starpu_free_on_node(unsigned dst_node, uintptr_t addr, size_t size)
 {
-	starpu_free_on_node_flags(dst_node, addr, size, malloc_on_node_default_flags[dst_node]);
+	starpu_free_on_node_flags(dst_node, addr, size, _starpu_get_node_struct(dst_node)->malloc_on_node_default_flags);
 }
