@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
- * Copyright (C) 2011       Télécom-SudParis
- * Copyright (C) 2013       Thibaut Lambert
+ * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2011	    Télécom-SudParis
+ * Copyright (C) 2013	    Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -27,6 +27,13 @@
 #include <core/sched_policy.h>
 #include <core/debug.h>
 #include <core/task.h>
+#include <datawizard/memory_nodes.h>
+#ifdef HAVE_MMAP
+#include <sys/mman.h>
+#endif
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <errno.h>
 
 
 void _starpu_driver_start_job(struct _starpu_worker *worker, struct _starpu_job *j, struct starpu_perfmodel_arch* perf_arch, int rank, int profiling)
@@ -36,33 +43,66 @@ void _starpu_driver_start_job(struct _starpu_worker *worker, struct _starpu_job
 	int workerid = worker->workerid;
 	unsigned calibrate_model = 0;
 
+	if (worker->bindid_requested != -1)
+	{
+		typedef unsigned __attribute__((__may_alias__)) alias_unsigned;
+		typedef int __attribute__((__may_alias__)) alias_int;
+
+		unsigned raw_bindid_requested = STARPU_VAL_EXCHANGE((alias_unsigned *)&worker->bindid_requested, -1);
+		int bindid_requested = *(alias_int *)&raw_bindid_requested;
+
+		if (bindid_requested != -1)
+		{
+			worker->bindid = bindid_requested;
+			_starpu_bind_thread_on_cpu(worker->bindid, worker->workerid, NULL);
+		}
+	}
 	if (cl->model && cl->model->benchmarking)
 		calibrate_model = 1;
 
 	/* If the job is executed on a combined worker there is no need for the
-	 * scheduler to process it : it doesn't contain any valuable data
+	 * scheduler to process on non-master : it doesn't contain any valuable data
 	 * as it's not linked to an actual worker */
 	if (j->task_size == 1 && rank == 0)
 		_starpu_sched_pre_exec_hook(task);
 
-	_starpu_set_worker_status(worker, STATUS_EXECUTING);
+	struct timespec start;
+
+	struct starpu_profiling_task_info *profiling_info = task->profiling_info;
+	if ((profiling && profiling_info) || (rank == 0 && (calibrate_model || !_starpu_perf_counter_paused())))
+		_starpu_clock_gettime(&start);
+	_starpu_add_worker_status(worker, STATUS_INDEX_EXECUTING, &start);
 
 	if (rank == 0)
 	{
 		STARPU_ASSERT(task->status == STARPU_TASK_READY);
+		if (!_starpu_perf_counter_paused() && !j->internal)
+		{
+			(void)STARPU_PERF_COUNTER_ADD64(& _starpu_task__g_current_ready__value, -1);
+			if (task->cl && task->cl->perf_counter_values)
+			{
+				struct starpu_perf_counter_sample_cl_values * const pcv = task->cl->perf_counter_values;
+				(void)STARPU_PERF_COUNTER_ADD64(&pcv->task.current_ready, -1);
+			}
+		}
 		task->status = STARPU_TASK_RUNNING;
 
 		STARPU_AYU_RUNTASK(j->job_id);
-		cl->per_worker_stats[workerid]++;
+		if (_starpu_codelet_profiling)
+			cl->per_worker_stats[workerid]++;
 
-		struct starpu_profiling_task_info *profiling_info = task->profiling_info;
-
-		if ((profiling && profiling_info) || calibrate_model)
+		if ((profiling && profiling_info) || calibrate_model || !_starpu_perf_counter_paused())
 		{
-			_starpu_clock_gettime(&worker->cl_start);
+			worker->cl_start = start;
+			if (profiling && profiling_info)
+			{
+				profiling_info->start_time = start;
+				profiling_info->workerid = workerid;
+			}
+
 			if (task->predicted && !isnan(task->predicted))
 			{
-				struct timespec exp_end = worker->cl_start;
+				struct timespec exp_end = start;
 				exp_end.tv_sec += task->predicted / 1000000;
 				exp_end.tv_nsec += fmod(task->predicted, 1000000.) * 1000;
 				if (exp_end.tv_nsec >= 1000000000)
@@ -77,8 +117,8 @@ void _starpu_driver_start_job(struct _starpu_worker *worker, struct _starpu_job
 				worker->cl_expend.tv_sec = 0;
 				worker->cl_expend.tv_nsec = 0;
 			}
-			_starpu_worker_register_executing_start_date(workerid, &worker->cl_start);
 		}
+
 		_starpu_job_notify_start(j, perf_arch);
 	}
 
@@ -108,12 +148,12 @@ void _starpu_driver_start_job(struct _starpu_worker *worker, struct _starpu_job
 				}
 			}
 		}
-		_STARPU_TRACE_TASK_COLOR(j);
+		_STARPU_TRACE_TASK_NAME_LINE_COLOR(j);
 		_STARPU_TRACE_START_CODELET_BODY(j, j->nimpl, &sched_ctx->perf_arch, workerid);
 	}
 	else
 	{
-		_STARPU_TRACE_TASK_COLOR(j);
+		_STARPU_TRACE_TASK_NAME_LINE_COLOR(j);
 		_STARPU_TRACE_START_CODELET_BODY(j, j->nimpl, perf_arch, workerid);
 	}
 	_starpu_sched_ctx_unlock_read(sched_ctx->id);
@@ -146,19 +186,19 @@ void _starpu_driver_end_job(struct _starpu_worker *worker, struct _starpu_job *j
 	if (cl && cl->model && cl->model->benchmarking)
 		calibrate_model = 1;
 
+	struct timespec end;
+	struct starpu_profiling_task_info *profiling_info = task->profiling_info;
+	if ((profiling && profiling_info) || (rank == 0 && (calibrate_model || !_starpu_perf_counter_paused())))
+		_starpu_clock_gettime(&end);
+	_starpu_clear_worker_status(worker, STATUS_INDEX_EXECUTING, &end);
+
 	if (rank == 0)
 	{
-		struct starpu_profiling_task_info *profiling_info = task->profiling_info;
-		if ((profiling && profiling_info) || calibrate_model)
-		{
-			_starpu_clock_gettime(&worker->cl_end);
-			_starpu_worker_register_executing_end(workerid);
-		}
+		if ((profiling && profiling_info) || calibrate_model || !_starpu_perf_counter_paused())
+			worker->cl_end = end;
 		STARPU_AYU_POSTRUNTASK(j->job_id);
 	}
 
-	_starpu_set_worker_status(worker, STATUS_UNKNOWN);
-
 	if(!sched_ctx->sched_policy && !sched_ctx->awake_workers &&
 	   sched_ctx->main_master == worker->workerid)
 	{
@@ -199,22 +239,32 @@ void _starpu_driver_update_job_feedback(struct _starpu_job *j, struct _starpu_wo
 		calibrate_model = 1;
 #endif
 
-	if ((profiling && profiling_info) || calibrate_model)
+	if ((profiling && profiling_info) || calibrate_model || !_starpu_perf_counter_paused())
 	{
-		double measured;
-
 		starpu_timespec_sub(&worker->cl_end, &worker->cl_start, &measured_ts);
-		measured = starpu_timing_timespec_to_us(&measured_ts);
+		double measured = starpu_timing_timespec_to_us(&measured_ts);
+
 		STARPU_ASSERT_MSG(measured >= 0, "measured=%lf\n", measured);
 
+		if (!_starpu_perf_counter_paused())
+		{
+			worker->__w_total_executed__value++;
+			worker->__w_cumul_execution_time__value += measured;
+			_starpu_perf_counter_update_per_worker_sample(worker->workerid);
+			if (cl->perf_counter_values)
+			{
+				struct starpu_perf_counter_sample_cl_values * const pcv = cl->perf_counter_values;
+				(void)STARPU_PERF_COUNTER_ADD64(&pcv->task.total_executed, 1);
+				_starpu_perf_counter_update_acc_double(&pcv->task.cumul_execution_time, measured);
+				_starpu_perf_counter_update_per_codelet_sample(cl);
+			}
+		}
+
 		if (profiling && profiling_info)
 		{
-			memcpy(&profiling_info->start_time, &worker->cl_start, sizeof(struct timespec));
-			memcpy(&profiling_info->end_time, &worker->cl_end, sizeof(struct timespec));
+			profiling_info->end_time = worker->cl_end;
 
-			profiling_info->workerid = workerid;
-
-			_starpu_worker_update_profiling_info_executing(workerid, &measured_ts, 1,
+			_starpu_worker_update_profiling_info_executing(workerid, 1,
 								       profiling_info->used_cycles,
 								       profiling_info->stall_cycles,
 								       profiling_info->energy_consumed,
@@ -249,18 +299,21 @@ void _starpu_driver_update_job_feedback(struct _starpu_job *j, struct _starpu_wo
 				do_update_time_model = 1;
 			}
 #else
-			const unsigned do_update_time_model = 1;
+			unsigned do_update_time_model = 1;
 			const double time_consumed = measured;
 #endif
+			if (j->task->failed)
+				/* Do not record perfmodel for failed tasks, they may terminate earlier */
+				do_update_time_model = 0;
 			if (do_update_time_model)
 			{
-				_starpu_update_perfmodel_history(j, j->task->cl->model, perf_arch, worker->devid, time_consumed, j->nimpl);
+				_starpu_update_perfmodel_history(j, j->task->cl->model, perf_arch, worker->devid, time_consumed, j->nimpl, 1);
 			}
 		}
 	}
 
 	if (!updated)
-		_starpu_worker_update_profiling_info_executing(workerid, NULL, 1, 0, 0, 0, 0);
+		_starpu_worker_update_profiling_info_executing(workerid, 1, 0, 0, 0, 0);
 
 	if (profiling_info && profiling_info->energy_consumed && cl->energy_model && cl->energy_model->benchmarking)
 	{
@@ -282,63 +335,52 @@ void _starpu_driver_update_job_feedback(struct _starpu_job *j, struct _starpu_wo
 		}
 #else
 		const double energy_consumed = profiling_info->energy_consumed;
-		const unsigned do_update_energy_model = 1;
+		unsigned do_update_energy_model = 1;
 #endif
 
+		if (j->task->failed)
+			/* Do not record perfmodel for failed tasks, they may terminate earlier */
+			do_update_energy_model = 0;
 		if (do_update_energy_model)
 		{
-			_starpu_update_perfmodel_history(j, j->task->cl->energy_model, perf_arch, worker->devid, energy_consumed, j->nimpl);
+			_starpu_update_perfmodel_history(j, j->task->cl->energy_model, perf_arch, worker->devid, energy_consumed, j->nimpl, 1);
 		}
 	}
 }
 
 static void _starpu_worker_set_status_scheduling(int workerid)
 {
-	if (_starpu_worker_get_status(workerid) == STATUS_SLEEPING)
+	if (!(_starpu_worker_get_status(workerid) & STATUS_SCHEDULING))
 	{
-		_starpu_worker_set_status(workerid, STATUS_SLEEPING_SCHEDULING);
-	}
-	else if (_starpu_worker_get_status(workerid) != STATUS_SCHEDULING)
-	{
-		_STARPU_TRACE_WORKER_SCHEDULING_START;
-		_starpu_worker_set_status(workerid, STATUS_SCHEDULING);
+		if (!(_starpu_worker_get_status(workerid) & STATUS_SLEEPING))
+			_STARPU_TRACE_WORKER_SCHEDULING_START;
+		_starpu_worker_add_status(workerid, STATUS_INDEX_SCHEDULING);
 	}
 }
 
 static void _starpu_worker_set_status_scheduling_done(int workerid)
 {
-	if (_starpu_worker_get_status(workerid) == STATUS_SLEEPING_SCHEDULING)
-	{
-		_starpu_worker_set_status(workerid, STATUS_SLEEPING);
-	}
-	else if (_starpu_worker_get_status(workerid) == STATUS_SCHEDULING)
-	{
+	STARPU_ASSERT(_starpu_worker_get_status(workerid) & STATUS_SCHEDULING);
+	if (!(_starpu_worker_get_status(workerid) & STATUS_SLEEPING))
 		_STARPU_TRACE_WORKER_SCHEDULING_END;
-		_starpu_worker_set_status(workerid, STATUS_UNKNOWN);
-	}
+	_starpu_worker_clear_status(workerid, STATUS_INDEX_SCHEDULING);
 }
 
 static void _starpu_worker_set_status_sleeping(int workerid)
 {
-	if (_starpu_worker_get_status(workerid) != STATUS_SLEEPING)
+	if (!(_starpu_worker_get_status(workerid) & STATUS_SLEEPING))
 	{
-		if (_starpu_worker_get_status(workerid) != STATUS_SLEEPING_SCHEDULING)
-		{
-			_STARPU_TRACE_WORKER_SLEEP_START;
-			_starpu_worker_restart_sleeping(workerid);
-		}
-		_starpu_worker_set_status(workerid, STATUS_SLEEPING);
+		_STARPU_TRACE_WORKER_SLEEP_START;
+		_starpu_worker_add_status(workerid, STATUS_INDEX_SLEEPING);
 	}
-
 }
 
 static void _starpu_worker_set_status_wakeup(int workerid)
 {
-	if (_starpu_worker_get_status(workerid) == STATUS_SLEEPING)
+	if ((_starpu_worker_get_status(workerid) & STATUS_SLEEPING))
 	{
 		_STARPU_TRACE_WORKER_SLEEP_END;
-		_starpu_worker_stop_sleeping(workerid);
-		_starpu_worker_set_status(workerid, STATUS_UNKNOWN);
+		_starpu_worker_clear_status(workerid, STATUS_INDEX_SLEEPING);
 	}
 }
 
@@ -686,3 +728,215 @@ int _starpu_get_multi_worker_task(struct _starpu_worker *workers, struct starpu_
 
 	return count;
 }
+
+#ifdef HAVE_MMAP
+/*generate and initialize rbtree map_tree*/
+static struct starpu_rbtree map_tree = STARPU_RBTREE_INITIALIZER;
+static starpu_pthread_mutex_t map_tree_mutex = STARPU_PTHREAD_MUTEX_INITIALIZER;
+
+struct map_allocate_info
+{
+	struct starpu_rbtree_node map_node;
+	void* map_addr;
+	size_t length;
+	char name[];
+};
+
+/* the cmp_fn arg for rb_tree_insert() */
+static unsigned int map_addr_cmp_insert(struct starpu_rbtree_node * left_elm, struct starpu_rbtree_node * right_elm)
+{
+	unsigned int addr_left = (uintptr_t)((struct map_allocate_info *) left_elm)->map_addr;
+	unsigned int addr_right = (uintptr_t)((struct map_allocate_info *) right_elm)->map_addr;
+
+	return addr_left - addr_right;
+}
+
+/* the cmp_fn arg for starpu_rbtree_lookup() */
+static unsigned int map_addr_cmp_lookup(uintptr_t addr_left, struct starpu_rbtree_node * right_elm)
+{
+	unsigned int addr_right = (uintptr_t)((struct map_allocate_info *) right_elm)->map_addr;
+
+	return addr_left - addr_right;
+}
+
+void *_starpu_map_allocate(size_t length, unsigned node)
+{
+	/*file*/
+	int fd;
+	char fd_name[32];
+	snprintf(fd_name,sizeof(fd_name), "starpu-%u-XXXXXX", node);
+
+	while(1)
+	{
+		mktemp(fd_name);
+		fd = shm_open(fd_name, O_RDWR|O_CREAT|O_EXCL, 0600);
+		if(fd >= 0)
+			break;
+		/* if name is already existed, recreate one*/
+		else if (errno == EEXIST)
+			continue;
+		else
+		{
+			perror("fail to open file");
+			return NULL;
+		}
+	}
+
+	/*fix the length of file*/
+	int ret = ftruncate(fd, length);
+	if (ret < 0)
+	{
+		perror("fail to allocate room for mapping");
+		close(fd);
+		return NULL;
+	}
+	void* map_addr = mmap(NULL, length, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0);
+	close(fd);
+	if (map_addr == MAP_FAILED)
+	{
+		perror("fail to map");
+		return NULL;
+	}
+
+	struct map_allocate_info *map_info;
+	_STARPU_MALLOC(map_info, sizeof(struct map_allocate_info)+strlen(fd_name)+1);
+	map_info->map_addr = map_addr;
+	map_info->length = length;
+	memcpy(map_info->name, fd_name, strlen(fd_name)+1);
+
+	starpu_rbtree_node_init(&map_info->map_node);
+
+	STARPU_PTHREAD_MUTEX_LOCK(&map_tree_mutex);
+	starpu_rbtree_insert(&map_tree, &map_info->map_node, map_addr_cmp_insert);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&map_tree_mutex);
+
+	return map_addr;
+}
+
+int _starpu_map_deallocate(void* map_addr, size_t length)
+{
+	STARPU_PTHREAD_MUTEX_LOCK(&map_tree_mutex);
+	struct starpu_rbtree_node * currentNode = starpu_rbtree_lookup(&map_tree, (uintptr_t)map_addr, map_addr_cmp_lookup);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&map_tree_mutex);
+
+	if (currentNode != NULL)
+	{
+		struct map_allocate_info * map_info = (struct map_allocate_info *) currentNode;
+		if ((uintptr_t)map_addr == (uintptr_t)map_info->map_addr && ((uintptr_t)map_addr + length) == ((uintptr_t)map_info->map_addr + map_info->length))
+		{
+			/*unlink the map fd name*/
+			if (shm_unlink(map_info->name) != 0)
+			{
+				_STARPU_DISP("warning: cannot unlink file %s: %s\n", map_info->name, strerror(errno));
+			}
+			STARPU_PTHREAD_MUTEX_LOCK(&map_tree_mutex);
+			starpu_rbtree_remove(&map_tree, &map_info->map_node);
+			STARPU_PTHREAD_MUTEX_UNLOCK(&map_tree_mutex);
+			free(map_info);
+		}
+		else
+		{
+			return -1;
+		}
+	}
+	else
+	{
+		_STARPU_DISP("could not find mapped address %p\n", map_addr);
+	}
+
+	int res = munmap(map_addr, length);
+	if (res < 0)
+	{
+		perror("fail to unmap");
+		return -1;
+	}
+
+	return 0;
+}
+
+/*lookup name from map_addr*/
+char* _starpu_get_fdname_from_mapaddr(uintptr_t map_addr, size_t *offset, size_t length)
+{
+	char* map_name = NULL;
+
+	STARPU_PTHREAD_MUTEX_LOCK(&map_tree_mutex);
+	struct starpu_rbtree_node * currentNode = starpu_rbtree_lookup_nearest(&map_tree, map_addr, map_addr_cmp_lookup, STARPU_RBTREE_LEFT);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&map_tree_mutex);
+
+	if (currentNode != NULL)
+	{
+		struct map_allocate_info * map_info = (struct map_allocate_info *) currentNode;
+
+		if ((map_addr >= (uintptr_t)map_info->map_addr) && map_addr + length <= ((uintptr_t)map_info->map_addr + map_info->length))
+		{
+			map_name = strdup(map_info->name);
+			*offset = map_addr - (uintptr_t)map_info->map_addr;
+		}
+	}
+
+	return map_name;
+}
+
+/*map with giving file name*/
+void *_starpu_sink_map(char *fd_name, size_t offset, size_t length)
+{
+	/*file*/
+	int fd;
+
+	fd = shm_open(fd_name, O_RDWR, 0600);
+
+	if(fd < 0)
+	{
+		perror("fail to open file");
+		return NULL;
+	}
+
+	/* offset for mmap() must be page aligned */
+	off_t pa_offset = offset & ~(sysconf(_SC_PAGE_SIZE) - 1);
+
+	void *map_sink_addr = mmap(NULL, length, PROT_READ|PROT_WRITE, MAP_SHARED, fd, pa_offset);
+	close(fd);
+	if (map_sink_addr == MAP_FAILED)
+	{
+		perror("fail to map");
+		return NULL;
+	}
+
+	return (void*)((uintptr_t)map_sink_addr + (offset - pa_offset));
+}
+
+int _starpu_sink_unmap(uintptr_t map_addr, size_t length)
+{
+	uintptr_t pa_addr = map_addr & ~(sysconf(_SC_PAGE_SIZE) - 1);
+	size_t offset = map_addr-pa_addr;
+
+	int res = munmap((void*)pa_addr, length + offset);
+	if (res < 0)
+	{
+		perror("fail to unmap");
+		return -1;
+	}
+	return 0;
+}
+#else
+char* _starpu_get_fdname_from_mapaddr(uintptr_t map_addr, size_t *offset, size_t length)
+{
+	(void)map_addr;
+	(void)offset;
+	(void)length;
+	return NULL;
+}
+void *_starpu_sink_map(char *fd_name, size_t offset, size_t length)
+{
+	(void)fd_name;
+	(void)offset;
+	(void)length;
+	return NULL;
+}
+int _starpu_sink_unmap(uintptr_t map_addr, size_t length)
+{
+	(void)map_addr;
+	(void)length;
+	return -1;
+}
+#endif
