@@ -22,7 +22,9 @@
 #endif
 #include <error.h>
 #include <errno.h>
+#ifdef MSG_ERRQUEUE
 #include <linux/errqueue.h>
+#endif
 #include <sys/types.h>
 #include <netdb.h>
 #include <arpa/inet.h>
@@ -108,9 +110,9 @@ struct _starpu_tcpip_ms_request
 	char* buf;
 	/*the length of message*/
 	int len;
-	/*a flag to detect wether the operation is completed*/
+	/*a flag to detect whether the operation is completed*/
 	int flag_completed;
-	/*a semaphore to detect wether the request is completed*/
+	/*a semaphore to detect whether the request is completed*/
 	starpu_sem_t sem_wait_request;
 	/*a flag to detect send or receive*/
 	int is_sender;
