@@ -1,7 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -18,14 +17,22 @@
 #include <starpu.h>
 #include "../helper.h"
 
+/*
+ * Test Dijkstra's Dining Philosophers problem
+ */
+
+/* TODO: try to use an arbiter and check improved concurrency */
+
 /* number of philosophers */
 #define N	16
 
 starpu_data_handle_t fork_handles[N];
 unsigned forks[N];
 
-static void eat_kernel(void *descr[], void *arg)
+void eat_kernel(void *descr[], void *arg)
 {
+	(void)descr;
+	(void)arg;
 }
 
 static struct starpu_codelet eating_cl =
@@ -34,9 +41,11 @@ static struct starpu_codelet eating_cl =
 	.cuda_funcs = {eat_kernel},
 	.cpu_funcs = {eat_kernel},
         .opencl_funcs = {eat_kernel},
+	.cpu_funcs_name = {"eat_kernel"},
 	.nbuffers = 2
 };
 
+static
 int submit_one_task(unsigned p)
 {
 	struct starpu_task *task = starpu_task_create();
@@ -57,7 +66,7 @@ int main(int argc, char **argv)
 {
 	int ret;
 
-	ret = starpu_init(NULL);
+	ret = starpu_initialize(NULL, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
@@ -67,10 +76,14 @@ int main(int argc, char **argv)
 	{
 		forks[f] = 0;
 
-		starpu_vector_data_register(&fork_handles[f], 0, (uintptr_t)&forks[f], 1, sizeof(unsigned));
+		starpu_vector_data_register(&fork_handles[f], STARPU_MAIN_RAM, (uintptr_t)&forks[f], 1, sizeof(unsigned));
+		starpu_data_set_sequential_consistency_flag(fork_handles[f], 0);
 	}
 
 	unsigned ntasks = 1024;
+#ifdef STARPU_HAVE_VALGRIND_H
+	if(RUNNING_ON_VALGRIND) ntasks = 3;
+#endif
 
 	unsigned t;
 	for (t = 0; t < ntasks; t++)
