@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2016                                Université de Bordeaux
- * Copyright (C) 2010-2013,2015                           CNRS
- * Copyright (C) 2013                                     Inria
+ * Copyright (C) 2009-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -49,6 +47,8 @@ static unsigned *v2;
 
 static void callback(void *arg)
 {
+	(void)arg;
+
 	unsigned res = STARPU_ATOMIC_ADD(&cnt, -1);
 	ANNOTATE_HAPPENS_BEFORE(&cnt);
 
@@ -62,18 +62,22 @@ static void callback(void *arg)
 	}
 }
 
-
-
-static void cuda_codelet_null(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+static void cuda_codelet_null(void *descr[], void *_args)
 {
+	(void)descr;
+	(void)_args;
 }
 
-static void opencl_codelet_null(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+static void opencl_codelet_null(void *descr[], void *_args)
 {
+	(void)descr;
+	(void)_args;
 }
 
-void cpu_codelet_null(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void cpu_codelet_null(void *descr[], void *_args)
 {
+	(void)descr;
+	(void)_args;
 }
 
 static enum starpu_data_access_mode select_random_mode(void)
@@ -193,8 +197,10 @@ int main(int argc, char **argv)
 
 	ret = starpu_malloc((void **)&v, VECTORSIZE*sizeof(unsigned));
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_malloc");
+	memset(v, 0, VECTORSIZE*sizeof(unsigned));
 	ret = starpu_malloc((void **)&v2, VECTORSIZE*sizeof(unsigned));
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_malloc");
+	memset(v2, 0, VECTORSIZE*sizeof(unsigned));
 
 	starpu_vector_data_register(&v_handle, STARPU_MAIN_RAM, (uintptr_t)v, VECTORSIZE, sizeof(unsigned));
 	starpu_vector_data_register(&v_handle2, STARPU_MAIN_RAM, (uintptr_t)v2, VECTORSIZE, sizeof(unsigned));
@@ -245,8 +251,8 @@ int main(int argc, char **argv)
 
 	starpu_data_unregister(v_handle);
 	starpu_data_unregister(v_handle2);
-	starpu_free(v);
-	starpu_free(v2);
+	starpu_free_noflag(v, VECTORSIZE*sizeof(unsigned));
+	starpu_free_noflag(v2, VECTORSIZE*sizeof(unsigned));
 	starpu_shutdown();
 
 	return EXIT_SUCCESS;
@@ -254,8 +260,8 @@ int main(int argc, char **argv)
 enodev:
 	starpu_data_unregister(v_handle);
 	starpu_data_unregister(v_handle2);
-	starpu_free(v);
-	starpu_free(v2);
+	starpu_free_noflag(v, VECTORSIZE*sizeof(unsigned));
+	starpu_free_noflag(v2, VECTORSIZE*sizeof(unsigned));
 	starpu_shutdown();
 	fprintf(stderr, "WARNING: No one can execute this task\n");
 	/* yes, we do not perform the computation but we did detect that no one
