@@ -1,9 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012                                     Inria
- * Copyright (C) 2012,2013,2015,2017                      CNRS
- * Copyright (C) 2013,2014,2016                           Université de Bordeaux
- * Copyright (C) 2013                                     Thibaut Lambert
+ * Copyright (C) 2012-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -29,14 +27,7 @@
 
 #define NTASKS           8
 
-void dummy(void *buffers[], void *args)
-{
-	(void) buffers;
-	(void) args;
-}
-
-static int
-run(struct starpu_sched_policy *p)
+static int run(struct starpu_sched_policy *p)
 {
 	int ret;
 	struct starpu_conf conf;
@@ -49,21 +40,12 @@ run(struct starpu_sched_policy *p)
 		exit(STARPU_TEST_SKIPPED);
 
 	struct starpu_task *tasks[NTASKS] = { NULL };
-	struct starpu_codelet cl =
-	{
-		.cpu_funcs    = {dummy},
-		.cpu_funcs_name = {"dummy"},
-		.cuda_funcs   = {dummy},
-		.opencl_funcs = {dummy},
-		.nbuffers     = 0
-	};
-
 	int i;
 	for (i = 0; i < NTASKS; i++)
 	{
 		struct starpu_task *task = starpu_task_create();
 		tasks[i] = task;
-		task->cl = &cl;
+		task->cl = &starpu_codelet_nop;
 		task->synchronous = 1;
 		task->destroy = 0;
 		ret = starpu_task_submit(task);
@@ -102,13 +84,15 @@ int main(void)
 	struct starpu_sched_policy **policies;
 	struct starpu_sched_policy **policy;
 
-#ifdef STARPU_HAVE_UNSETENV
-	unsetenv("STARPU_SCHED");
-#endif
+	char *sched = getenv("STARPU_SCHED");
 
 	policies = starpu_sched_get_predefined_policies();
 	for(policy=policies ; *policy!=NULL ; policy++)
 	{
+		if (sched && strcmp(sched, (*policy)->policy_name))
+			/* Testing another specific scheduler, no need to run this */
+			continue;
+
 		FPRINTF(stderr, "Running with policy %s.\n", (*policy)->policy_name);
 		int ret;
 		ret = run(*policy);
