@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2012-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -140,6 +140,7 @@ static struct starpu_codelet cpu_cl =
 	.cpu_funcs    = { dummy },
 	.cuda_funcs   = { dummy },
 	.opencl_funcs = { dummy },
+	.max_fpga_funcs   = { dummy },
 	.nbuffers     = 0,
 	.model        = &model_cpu_task
 };
@@ -149,6 +150,7 @@ static struct starpu_codelet gpu_cl =
 	.cpu_funcs    = { dummy },
 	.cuda_funcs   = { dummy },
 	.opencl_funcs = { dummy },
+	.max_fpga_funcs   = { dummy },
 	.nbuffers     = 0,
 	.model        = &model_gpu_task
 };
@@ -262,7 +264,7 @@ int main(void)
 
 	char *sched = getenv("STARPU_SCHED");
 
-	if (starpu_get_env_number_default("STARPU_NWORKER_PER_CUDA", 1) != 1)
+	if (starpu_getenv_number_default("STARPU_NWORKER_PER_CUDA", 1) != 1)
 		return STARPU_TEST_SKIPPED;
 
 	int i;
